#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class ConfigMapEnvSource(object):
    """
    https://kubernetes.io/docs/api-reference/v1.8/#configmapenvsource-v1-core

    ConfigMapEnvSource selects a ConfigMap to populate the environment variables with.

    The contents of the target ConfigMap's Data field will represent the key-value pairs as environment variables.
    """

    def __init__(self, model=None):
        super(ConfigMapEnvSource, self).__init__()

        self._name = None
        self._optional = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']
        if 'optional' in model:
            self.optional = model['optional']

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ConfigMapVolumeSource: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- optional

    @property
    def optional(self):
        return self._optional

    @optional.setter
    def optional(self, v=None):
        if not isinstance(v, bool):
            raise SyntaxError('ConfigMapVolumeSource: optional: [ {0} ] is invalid.'.format(v))
        self._optional = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name is not None:
            data['name'] = self.name
        if self.optional is not None:
            data['optional'] = self.optional
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_list, is_valid_dict, is_valid_string


class HTTPGetAction(object):

    VALID_SCHEMES = ['HTTP', 'HTTPS']

    def __init__(self, model=None):
        super(HTTPGetAction, self).__init__()

        self._http_headers = []
        self._scheme = 'HTTP'
        self._host = None
        self._path = None
        self._port = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'path' in model:
            self.path = model['path']
        if 'port' in model:
            self.port = model['port']
        if 'host' in model:
            self.host = model['host']
        if 'scheme' in model:
            self.scheme = model['scheme']
        if 'httpHeaders' in model:
            self.http_headers = model['httpHeaders']

    # ------------------------------------------------------------------------------------- add

    def add_header(self, name=None, value=None):
        if not is_valid_string(name) or not is_valid_string(value):
            raise SyntaxError('HTTPGetAction: header: [ {0} : {1} ] is invalid.'.format(name, value))
        data = {'name': name, 'value': value}
        self._http_headers.append(data)

    # ------------------------------------------------------------------------------------- path

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, path=None):
        if not is_valid_string(path):
            raise SyntaxError('HTTPGetAction: path: [ {0} ] is invalid.'.format(path))
        self._path = path

    # ------------------------------------------------------------------------------------- port

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, port=None):
        msg = 'HTTPGetAction: port: [ {0} ] is invalid.'.format(port)
        if isinstance(port, str) and port.isdigit():
            port = int(port)
        if isinstance(port, int) and not 1 < port < 65535:
            raise SyntaxError(msg)
        self._port = port

    # ------------------------------------------------------------------------------------- host

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, host=None):
        if not is_valid_string(host) :
            raise SyntaxError('HTTPGetAction: host: [ {0} ] is invalid.'.format(host))
        self._host = host

    # ------------------------------------------------------------------------------------- scheme

    @property
    def scheme(self):
        return self._scheme

    @scheme.setter
    def scheme(self, scheme=None):
        if scheme not in HTTPGetAction.VALID_SCHEMES:
            raise SyntaxError('HTTPGetAction: scheme: [ {0} ] is invalid.'.format(scheme))
        self._scheme = scheme

    # ------------------------------------------------------------------------------------- http headers

    @property
    def http_headers(self):
        return self._http_headers

    @http_headers.setter
    def http_headers(self, headers=None):
        msg = 'HTTPGetAction: headers: [ {0} ] is invalid.'.format(headers)
        if not is_valid_list(headers, dict):
            raise SyntaxError(msg)
        for x in headers:
            if not is_valid_dict(x, ['name', 'value']):
                raise SyntaxError(msg)
        self._http_headers = headers

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.path is not None:
            data['path'] = self.path
        if self.port is not None:
            data['port'] = self.port
        if self.host is not None:
            data['host'] = self.host
        if self.scheme:
            data['scheme'] = self.scheme
        if self.http_headers:
            data['httpHeaders'] = self.http_headers
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import time

from kubernetes.K8sConfig import K8sConfig
from kubernetes.K8sContainer import K8sContainer
from kubernetes.K8sExceptions import BadRequestException
from kubernetes.K8sExceptions import TimedOutException, NotFoundException
from kubernetes.K8sObject import K8sObject
from kubernetes.K8sVolume import K8sVolume
from kubernetes.K8sPod import K8sPod
from kubernetes.K8sReplicaSet import K8sReplicaSet
from kubernetes.models.v1beta1.Deployment import Deployment
from kubernetes.models.v1beta1.DeploymentRollback import DeploymentRollback
from kubernetes.models.v1beta1.RollbackConfig import RollbackConfig
from kubernetes.models.v1beta1.LabelSelector import LabelSelector
from kubernetes.utils import is_valid_list


class K8sDeployment(K8sObject):

    SCALE_WAIT_TIMEOUT_SECONDS = 120
    REVISION_ANNOTATION = 'deployment.kubernetes.io/revision'

    def __init__(self, config=None, name=None, image=None, replicas=0):

        super(K8sDeployment, self).__init__(
            config=config,
            obj_type='Deployment',
            name=name
        )

        self.desired_replicas = replicas

        labels = {'name': name}
        sel = LabelSelector()
        sel.match_labels = labels

        self.selector = sel
        self.labels = labels
        self.pod_labels = labels

        if image is not None:
            container = K8sContainer(name=name, image=image)
            self.add_container(container)

        if self.config.pull_secret is not None:
            self.add_image_pull_secrets(self.config.pull_secret)

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sDeployment, self).create()
        self.get()
        if self.desired_replicas > 0:
            self._wait_for_desired_replicas()
        return self

    def update(self):
        super(K8sDeployment, self).update()
        self.get()
        if self.desired_replicas > 0:
            self._wait_for_desired_replicas()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sDeployment, self).list(labels=labels)
        deploys = list(map(lambda d: Deployment(d), ls))
        if pattern is not None:
            deploys = list(filter(lambda dep: pattern in dep.name, deploys))
        k8s = list()
        for x in deploys:
            j = K8sDeployment(config=self.config, name=x.name).from_model(m=x)
            k8s.append(j)
        return k8s

    def delete(self, cascade=False):
        # delete cascade on top level
        super(K8sDeployment, self).delete(cascade)
        if cascade:
            rsets = K8sReplicaSet(
                config=self.config,
                name="yo"
            ).list(pattern=self.name)
            # delete cascade on replicasets
            for rset in rsets:
                try:
                    rset.delete(cascade=cascade)
                except NotFoundException:
                    pass
            pods = K8sPod(
                config=self.config,
                name="yo"
            ).list(pattern=self.name)
            # delete cascade on pods
            for pod in pods:
                try:
                    pod.delete(cascade=cascade)
                except NotFoundException:
                    pass
        return self

    # -------------------------------------------------------------------------------------  wait

    def _wait_for_desired_replicas(self):
        start_time = time.time()
        while not self._has_desired_replicas():
            time.sleep(0.5)
            self.get()
            self._check_timeout(start_time)

    def _has_desired_replicas(self):
        if self.updated_replicas == self.desired_replicas \
                and self.current_replicas == self.desired_replicas \
                and self.available_replicas == self.desired_replicas:
            return True
        return False

    def _check_timeout(self, start_time=None):
        elapsed_time = time.time() - start_time
        if elapsed_time >= self.SCALE_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException(
                "Timed out scaling Deployment: [ {} ] to replica count: [ {} ]".format(
                    self.name,
                    self.desired_replicas))

    # -------------------------------------------------------------------------------------  add

    def add_container(self, container=None):
        if not isinstance(container, K8sContainer):
            raise SyntaxError('K8sDeployment.add_container() container: [ {0} ] is invalid.'.format(container))
        containers = self.model.spec.template.spec.containers
        if container.model not in containers:
            containers.append(container.model)
        self.model.spec.template.spec.containers = containers
        return self

    def add_image_pull_secrets(self, secret=None):
        self.model.spec.template.spec.add_image_pull_secrets(secret)
        return self

    def add_volume(self, volume=None):
        if not isinstance(volume, K8sVolume):
            raise SyntaxError('K8sDeployment.add_volume() volume: [ {0} ] is invalid.'.format(volume))
        volumes = self.model.spec.template.spec.volumes
        if volume.model not in volumes:
            volumes.append(volume.model)
        self.model.spec.template.spec.volumes = volumes
        return self

    # -------------------------------------------------------------------------------------  get

    def get(self):
        self.model = Deployment(self.get_model())
        return self

    # ------------------------------------------------------------------------------------- namespace

    @property
    def namespace(self):
        return self.model.metadata.namespace

    @namespace.setter
    def namespace(self, nspace=None):
        self.model.metadata.namespace = nspace

    # -------------------------------------------------------------------------------------  pod annotations

    @property
    def pod_annotations(self):
        return self.model.spec.template.metadata.annotations

    @pod_annotations.setter
    def pod_annotations(self, anns=None):
        self.model.spec.template.metadata.annotations = anns

    # -------------------------------------------------------------------------------------  pod labels

    @property
    def pod_labels(self):
        return self.model.spec.template.metadata.labels

    @pod_labels.setter
    def pod_labels(self, labels=None):
        self.model.spec.template.metadata.labels = labels

    # ------------------------------------------------------------------------------------- current replicas

    @property
    def current_replicas(self):
        return self.model.status.replicas

    @current_replicas.setter
    def current_replicas(self, reps=None):
        self.model.status.replicas = reps

    # ------------------------------------------------------------------------------------- desired replicas

    @property
    def desired_replicas(self):
        return self.model.spec.replicas

    @desired_replicas.setter
    def desired_replicas(self, reps=None):
        self.model.spec.replicas = reps

    # ------------------------------------------------------------------------------------- updated replicas

    @property
    def updated_replicas(self):
        return self.model.status.updated_replicas

    @updated_replicas.setter
    def updated_replicas(self, reps=None):
        self.model.status.updated_replicas = reps

    # ------------------------------------------------------------------------------------- available replicas

    @property
    def available_replicas(self):
        return self.model.status.available_replicas

    @available_replicas.setter
    def available_replicas(self, reps=None):
        self.model.status.available_replicas = reps

    # ------------------------------------------------------------------------------------- unavailable replicas

    @property
    def unavailable_replicas(self):
        return self.model.status.unavailable_replicas

    @unavailable_replicas.setter
    def unavailable_replicas(self, reps=None):
        self.model.status.unavailable_replicas = reps

    # -------------------------------------------------------------------------------------  revision

    @property
    def revision(self):
        if 'deployment.kubernetes.io/revision' in self.annotations:
            return int(self.annotations['deployment.kubernetes.io/revision'])
        return None

    @revision.setter
    def revision(self, r=None):
        raise NotImplementedError("K8sDeployment: revision is read-only.")

    # -------------------------------------------------------------------------------------  selector

    @property
    def selector(self):
        return self.model.spec.selector

    @selector.setter
    def selector(self, sel=None):
        self.model.spec.selector = sel

    # -------------------------------------------------------------------------------------  nodeSelector

    @property
    def node_selector(self):
        return self.model.spec.template.spec.node_selector

    @node_selector.setter
    def node_selector(self, sel=None):
        self.model.spec.template.spec.node_selector = sel

    # -------------------------------------------------------------------------------------  containers

    @property
    def containers(self):
        objs = []
        for c in self.model.spec.template.spec.containers:
            k8scontainer = K8sContainer(name=c.name, image=c.image)
            k8scontainer.model = c
            objs.append(k8scontainer)
        return objs

    @containers.setter
    def containers(self, containers=None):
        models = []
        for obj in containers:
            models.append(obj.model)
        self.model.spec.template.spec.containers = models

    # -------------------------------------------------------------------------------------  container_image

    @property
    def container_image(self, name=None):
        if name is None and len(self.containers) > 1:
            raise SyntaxError("K8sDeployment.container_image() Please specify a container name.")

        if len(self.containers) == 1:
            return self.containers[0].image
        else:
            filtered = list(filter(lambda x: x.name == name, self.containers))
            if filtered:
                return filtered[0].image
            return None

    @container_image.setter
    def container_image(self, tup=None):
        if not isinstance(tup, tuple):
            raise SyntaxError('K8sDeployment.container_image() must be a tuple of the form (name, image)')
        name, image = tup
        found = list(filter(lambda x: x.name == name, self.containers))
        if found:
            new = list(filter(lambda x: x.name != name, self.containers))
            found[0].image = image
            new.append(found[0])
            self.containers = new

    # -------------------------------------------------------------------------------------  volumes

    @property
    def volumes(self):
        return self.model.spec.template.spec.volumes

    @volumes.setter
    def volumes(self, v=None):
        if not is_valid_list(v, K8sVolume):
            self.model.spec.template.spec.volumes = v

    # -------------------------------------------------------------------------------------  get by name

    @staticmethod
    def get_by_name(config=None, name=None, name_label='name'):
        """
        Fetches a K8sDeployment by name.
        
        :param config: A K8sConfig object.
        :param name: The name we want.
        :param name_label: The label key to use for name.
        :return: A list of K8sDeployment objects.
        """

        if name is None:
            raise SyntaxError(
                'Deployment: name: [ {0} ] cannot be None.'.format(name))
        if not isinstance(name, str):
            raise SyntaxError(
                'Deployment: name: [ {0} ] must be a string.'.format(name))

        if config is not None and not isinstance(config, K8sConfig):
            raise SyntaxError(
                'Deployment: config: [ {0} ] must be a K8sConfig'.format(config))

        deps = K8sDeployment(config=config, name=name).list(labels={
            name_label: name
        })

        return deps

    # -------------------------------------------------------------------------------------  rollback

    def rollback(self, revision=None, annotations=None):
        """
        Performs a rollback of the Deployment.

        If the 'revision' parameter is omitted, we fetch the Deployment's system-generated
        annotation containing the current revision, and revert to the version immediately
        preceding the current version.

        :param revision: The revision to rollback to.
        :param annotations: Annotations we'd like to update.
        :return: self
        """

        rollback = DeploymentRollback()
        rollback.name = self.name

        rollback_config = RollbackConfig()

        # to the specified revision
        if revision is not None:
            rollback_config.revision = revision
        # to the revision immediately preceding the current revision
        else:
            current_revision = int(self.get_annotation(self.REVISION_ANNOTATION))
            rev = max(current_revision - 1, 0)
            rollback_config.revision = rev

        rollback.rollback_to = rollback_config

        if annotations is not None:
            rollback.updated_annotations = annotations

        url = '{base}/{name}/rollback'.format(base=self.base_url, name=self.name)
        state = self.request(
            method='POST',
            url=url,
            data=rollback.serialize())

        if not state.get('success'):
            status = state.get('status', '')
            reason = state.get('data', dict()).get('message', None)
            message = 'K8sDeployment: ROLLBACK failed : HTTP {0} : {1}'.format(status, reason)
            raise BadRequestException(message)

        time.sleep(0.2)
        self._wait_for_desired_replicas()
        self.get()

        return self

    # -------------------------------------------------------------------------------------  scale

    def scale(self, replicas=None):
        """
        Scales up or down.
        
        :param replicas: The number of desired replicas.
        :return: self
        """

        self.desired_replicas = replicas
        self.update()
        return self

    # -------------------------------------------------------------------------------------  purge replica sets

    def purge_replica_sets(self, keep=3):
        """
        Builds a list of ReplicaSets, sorted from newest to oldest.
        Slices the array, keeping the most X most recent ReplicaSets.
        
        :param keep: The number of ReplicaSets to keep.
        :return: None
        """

        rsets = K8sReplicaSet(
            config=self.config,
            name="yo"
        ).list(pattern=self.name, reverse=True)

        to_purge = rsets[keep:]
        for rset in to_purge:
            rset.delete(cascade=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class SecretKeySelector(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_secretkeyselector
    """

    def __init__(self, model=None):
        super(SecretKeySelector, self).__init__()

        self._name = None
        self._key = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']
        if 'key' in model:
            self.key = model['key']

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, n=None):
        if not is_valid_string(n):
            raise SyntaxError('SecretKeySelector: name: [ {} ] is invalid.'.format(n))
        self._name = n

    # ------------------------------------------------------------------------------------- key

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('SecretKeySelector: key: [ {} ] is invalid.'.format(k))
        self._key = k

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name is not None:
            data['name'] = self.name
        if self.key is not None:
            data['key'] = self.key
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.utils import filter_model


class NodeAddress(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nodeaddress
    """

    def __init__(self, model=None):
        super(NodeAddress, self).__init__()

        self._type = None
        self._address = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'type' in model:
            self.type = model['type']
        if 'address' in model:
            self.address = model['address']

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeAddress: type: [ {0} ] is invalid.'.format(v))
        self._type = v

    # ------------------------------------------------------------------------------------- address

    @property
    def address(self):
        return self._address

    @address.setter
    def address(self, v):
        if not isinstance(v, str):
            raise SyntaxError('NodeAddress: address: [ {0} ] is invalid.'.format(v))
        self._address = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.type:
            data['type'] = self.type
        if self.address:
            data['address'] = self.address
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class AWSElasticBlockStoreVolumeSource(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_awselasticblockstorevolumesource

    An AWS EBS disk with the specified volume-id must exist before mounting to a container.
    """

    def __init__(self, model=None):
        super(AWSElasticBlockStoreVolumeSource, self).__init__()

        self._fs_type = None
        self._partition = None
        self._read_only = None
        self._volume_id = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'fsType' in model:
            self.fs_type = model['fsType']
        if 'partition' in model:
            self.partition = model['partition']
        if 'readOnly' in model:
            self.read_only = model['readOnly']
        if 'volumeID' in model:
            self.volume_id = model['volumeID']

    # ------------------------------------------------------------------------------------- fs type

    @property
    def fs_type(self):
        return self._fs_type

    @fs_type.setter
    def fs_type(self, fs=None):
        if not is_valid_string(fs):
            raise SyntaxError('AWSElasticBlockStoreVolumeSource: fs_type: [ {0} ] is invalid.'.format(fs))
        self._fs_type = fs

    # ------------------------------------------------------------------------------------- partition

    @property
    def partition(self):
        return self._partition

    @partition.setter
    def partition(self, partition=None):
        if not isinstance(partition, int):
            raise SyntaxError('AWSElasticBlockStoreVolumeSource: partition: [ {0} ] is invalid.'.format(partition))
        self._partition = partition

    # ------------------------------------------------------------------------------------- read only

    @property
    def read_only(self):
        return self._read_only

    @read_only.setter
    def read_only(self, ro=None):
        if not isinstance(ro, bool):
            raise SyntaxError('AWSElasticBlockStoreVolumeSource: read_only: [ {0} ] is invalid.'.format(ro))
        self._read_only = ro

    # ------------------------------------------------------------------------------------- volume id

    @property
    def volume_id(self):
        return self._volume_id

    @volume_id.setter
    def volume_id(self, vid=None):
        if not is_valid_string(vid):
            raise SyntaxError('AWSElasticBlockStoreVolumeSource: volume_id: [ {0} ] is invalid.'.format(vid))
        self._volume_id = vid

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.fs_type is not None:
            data['fsType'] = self.fs_type
        if self.partition is not None:
            data['partition'] = self.partition
        if self.read_only is not None:
            data['readOnly'] = self.read_only
        if self.volume_id is not None:
            data['volumeID'] = self.volume_id
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json

import yaml

from kubernetes.models.v1.Volume import Volume


class K8sVolume(object):
    VALID_VOLUME_TYPES = Volume.VOLUME_TYPES_TO_SOURCE_MAP.keys()

    def __init__(self, name=None, type=None):
        super(K8sVolume, self).__init__()
        self._type = None
        self.model = Volume()
        self.name = name
        self.type = type

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.name

    @name.setter
    def name(self, name=None):
        self.model.name = name

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, t=None):
        if t not in self.VALID_VOLUME_TYPES:
            raise SyntaxError('K8sVolume: type: [ {} ] is invalid.'.format(t))
        self._type = t
        setattr(self.model, t, Volume.vol_type_to_source(t))

    # ------------------------------------------------------------------------------------- source

    @property
    def source(self):
        return getattr(self.model, self._type, None)

    @source.setter
    def source(self, s=None):
        raise NotImplementedError()

    # ------------------------------------------------------------------------------------- medium (emptyDir)

    @property
    def medium(self):
        if not hasattr(self.source, 'medium'):
            raise NotImplementedError()
        return self.source.medium

    @medium.setter
    def medium(self, m=None):
        if not hasattr(self.source, 'medium'):
            raise NotImplementedError()
        self.source.medium = m

    # ------------------------------------------------------------------------------------- path (hostPath)

    @property
    def path(self):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        return self.source.path

    @path.setter
    def path(self, p=None):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        self.source.path = p

    # ------------------------------------------------------------------------------------- secret_name (secret)

    @property
    def secret_name(self):
        if not hasattr(self.source, 'secret_name'):
            raise NotImplementedError()
        return self.source.secret_name

    @secret_name.setter
    def secret_name(self, sn=None):
        if not hasattr(self.source, 'secret_name'):
            raise NotImplementedError()
        self.source.secret_name = sn

    # ------------------------------------------------------------------------------------- volume_id (AWS)

    # http://kubernetes.io/docs/user-guide/volumes/#awselasticblockstore
    # - the nodes on which pods are running must be AWS EC2 instances
    # - those instances need to be in the same region and availability-zone as the EBS volume
    # - EBS only supports a single EC2 instance mounting a volume

    # Pod creation will timeout waiting for readiness if not on AWS; unschedulable.

    @property
    def volume_id(self):
        if not hasattr(self.source, 'volume_id'):
            raise NotImplementedError()
        return self.source.volume_id

    @volume_id.setter
    def volume_id(self, vid=None):
        if not hasattr(self.source, 'volume_id'):
            raise NotImplementedError()
        self.source.volume_id = vid

    # ------------------------------------------------------------------------------------- pd_name (GCE)

    # http://kubernetes.io/docs/user-guide/volumes/#gcepersistentdisk
    # - the nodes on which pods are running must be GCE VMs
    # - those VMs need to be in the same GCE project and zone as the PD

    # Pod creation will timeout waiting for readiness if not on GCE; unschedulable.

    @property
    def pd_name(self):
        if not hasattr(self.source, 'pd_name'):
            raise NotImplementedError()
        return self.source.pd_name

    @pd_name.setter
    def pd_name(self, pd=None):
        if not hasattr(self.source, 'pd_name'):
            raise NotImplementedError()
        self.source.pd_name = pd

    # ------------------------------------------------------------------------------------- read_only (GCE)

    # HTTP 422: GCE PD can only be mounted on multiple machines if it is read-only

    @property
    def read_only(self):
        if not hasattr(self.source, 'read_only'):
            raise NotImplementedError()
        return self.source.read_only

    @read_only.setter
    def read_only(self, ro=None):
        if not hasattr(self.source, 'read_only'):
            raise NotImplementedError()
        self.source.read_only = ro

    # ------------------------------------------------------------------------------------- fs_type (AWS, GCE)

    @property
    def fs_type(self):
        if not hasattr(self.source, 'fs_type'):
            raise NotImplementedError()
        return self.source.fs_type

    @fs_type.setter
    def fs_type(self, t=None):
        if not hasattr(self.source, 'fs_type'):
            raise NotImplementedError()
        self.source.fs_type = t

    # ------------------------------------------------------------------------------------- nfs_server

    @property
    def nfs_server(self):
        if not hasattr(self.source, 'server'):
            raise NotImplementedError()
        return self.source.server

    @nfs_server.setter
    def nfs_server(self, s=None):
        if not hasattr(self.source, 'server'):
            raise NotImplementedError()
        self.source.server = s

    # ------------------------------------------------------------------------------------- nfs_path

    @property
    def nfs_path(self):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        return self.source.path

    @nfs_path.setter
    def nfs_path(self, p=None):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        self.source.path = p

    # ------------------------------------------------------------------------------------- git_repository

    @property
    def git_repository(self):
        if not hasattr(self.source, 'repository'):
            raise NotImplementedError()
        return self.source.repository

    @git_repository.setter
    def git_repository(self, repo=None):
        if not hasattr(self.source, 'repository'):
            raise NotImplementedError()
        self.source.repository = repo

    # ------------------------------------------------------------------------------------- git_revision

    @property
    def git_revision(self):
        if not hasattr(self.source, 'revision'):
            raise NotImplementedError()
        return self.source.revision

    @git_revision.setter
    def git_revision(self, rev=None):
        if not hasattr(self.source, 'revision'):
            raise NotImplementedError()
        self.source.revision = rev

    # ------------------------------------------------------------------------------------- claimName

    @property
    def claim_name(self):
        if not hasattr(self.source, 'claim_name'):
            raise NotImplementedError()
        return self.source.claim_name

    @claim_name.setter
    def claim_name(self, name=None):
        if not hasattr(self.source, 'claim_name'):
            raise NotImplementedError()
        self.source.claim_name = name

    # ------------------------------------------------------------------------------------- configmap_name

    @property
    def configmap_name(self):
        if not hasattr(self.source, 'name'):
            raise NotImplementedError()
        return self.source.name

    @configmap_name.setter
    def configmap_name(self, name=None):
        if not hasattr(self.source, 'name'):
            raise NotImplementedError()
        self.source.name = name

    # ------------------------------------------------------------------------------------- configmap_items

    @property
    def configmap_items(self):
        if not hasattr(self.source, 'items'):
            raise NotImplementedError()
        return self.source.items

    @configmap_items.setter
    def configmap_items(self, v=None):
        if not hasattr(self.source, 'items'):
            raise NotImplementedError()
        self.source.items = v

    # ------------------------------------------------------------------------------------- configmap_default_mode

    @property
    def configmap_default_mode(self):
        if not hasattr(self.source, 'default_mode'):
            raise NotImplementedError()
        return self.source.default_mode

    @configmap_default_mode.setter
    def configmap_default_mode(self, v=None):
        if not hasattr(self.source, 'default_mode'):
            raise NotImplementedError()
        self.source.default_mode = v

    # ------------------------------------------------------------------------------------- configmap_optional

    @property
    def configmap_optional(self):
        if not hasattr(self.source, 'optional'):
            raise NotImplementedError()
        return self.source.optional

    @configmap_optional.setter
    def configmap_optional(self, v=None):
        if not hasattr(self.source, 'optional'):
            raise NotImplementedError()
        self.source.optional = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        return self.model.serialize()

    def as_json(self):
        data = self.serialize()
        dump = json.dumps(data, indent=4)
        return dump

    def as_yaml(self):
        data = self.serialize()
        dump = yaml.dump(data, default_flow_style=False)
        return dump



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.utils import filter_model


class DaemonEndpoint(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_daemonendpoint
    """

    def __init__(self, model=None):
        super(DaemonEndpoint, self).__init__()

        self._port = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'Port' in model:
            self.port = model['Port']

    # ------------------------------------------------------------------------------------- port

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, v):
        if not isinstance(v, int):
            raise SyntaxError('DaemonEndpoints: port: [ {0} ] is invalid.'.format(v))
        self._port = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.port:
            data['Port'] = self.port
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1beta1.LabelSelector import LabelSelector
from kubernetes.utils import is_valid_string, is_valid_list, convert


class PodAffinityTerm(object):

    def __init__(self, model=None):
        super(PodAffinityTerm, self).__init__()

        self._label_selector = None
        self._namespaces = []
        self._topology_key = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'labelSelector' in model:
            self.label_selector = LabelSelector(model['labelSelector'])
        if 'namespaces' in model:
            self.namespaces = model['namespaces']
        if 'topologyKey' in model:
            self.topology_key = model['topologyKey']

    # ------------------------------------------------------------------------------------- labelSelector

    @property
    def label_selector(self):
        return self._label_selector

    @label_selector.setter
    def label_selector(self, s=None):
        if not isinstance(s, LabelSelector):
            raise SyntaxError('PodAffinityTerm: label_selector: [ {} ] is invalid.'.format(s))
        self._label_selector = s

    # ------------------------------------------------------------------------------------- namespaces

    @property
    def namespaces(self):
        return self._namespaces

    @namespaces.setter
    def namespaces(self, n=None):
        if not is_valid_list(convert(n), str):
            raise SyntaxError('PodAffinityTerm: namespaces: [ {} ] is invalid.'.format(n))
        self._namespaces = n

    # ------------------------------------------------------------------------------------- topologyKey

    @property
    def topology_key(self):
        return self._topology_key

    @topology_key.setter
    def topology_key(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('PodAffinityTerm: topology_key: [ {} ] is invalid.'.format(t))
        self._topology_key = t

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.label_selector is not None:
            data['labelSelector'] = self.label_selector.serialize()
        if self.namespaces:
            data['namespaces'] = self.namespaces
        if self.topology_key is not None:
            data['topologyKey'] = self.topology_key
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PodTemplateSpec import PodTemplateSpec
from kubernetes.models.v1beta1.LabelSelector import LabelSelector


class JobSpec(object):
    """
    http://kubernetes.io/docs/api-reference/batch/v1/definitions/#_v1_jobspec
    """

    VALID_RESTART_POLICIES = ['OnFailure', 'Never']

    def __init__(self, model=None):
        super(JobSpec, self).__init__()

        self._parallelism = None
        self._completions = None
        self._active_deadline_seconds = None
        self._selector = None
        self._manual_selector = None
        self._template = PodTemplateSpec()

        self.template.spec.VALID_RESTART_POLICIES = JobSpec.VALID_RESTART_POLICIES
        if self.template.spec.restart_policy not in JobSpec.VALID_RESTART_POLICIES:
            self.template.spec.restart_policy = 'OnFailure'

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'parallelism' in model:
            self.parallelism = model['parallelism']
        if 'completions' in model:
            self.completions = model['completions']
        if 'activeDeadlineSeconds' in model:
            self.active_deadline_seconds = model['activeDeadlineSeconds']
        if 'selector' in model:
            self.selector = LabelSelector(model['selector'])
        if 'manualSelector' in model:
            self.manual_selector = model['manualSelector']
        if 'template' in model:
            self.template = PodTemplateSpec(model['template'])

    # --------------------------------------------------------------------------------- parallelism

    # .parallelism can be set to any non-negative value. If it is unspecified, it defaults to 1.
    #  If it is specified as 0, then the Job is effectively paused until it is increased.

    @property
    def parallelism(self):
        return self._parallelism

    @parallelism.setter
    def parallelism(self, p=None):
        if not isinstance(p, int) or not p >= 0:
            raise SyntaxError('JobSpec: parallelism: [ {} ] is invalid.'.format(p))
        self._parallelism = p

    # --------------------------------------------------------------------------------- completions

    @property
    def completions(self):
        return self._completions

    @completions.setter
    def completions(self, c=None):
        if not isinstance(c, int) or not c >= 0:
            raise SyntaxError('JobSpec: completions: [ {} ] is invalid.'.format(c))
        self._completions = c

    # --------------------------------------------------------------------------------- activeDeadlineSeconds

    @property
    def active_deadline_seconds(self):
        return self._active_deadline_seconds

    @active_deadline_seconds.setter
    def active_deadline_seconds(self, ads=None):
        if not isinstance(ads, int) or not ads >= 0:
            raise SyntaxError('JobSpec: active_deadline_seconds: [ {} ] is invalid.'.format(ads))
        self._active_deadline_seconds = ads

    # --------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self._selector

    @selector.setter
    def selector(self, s=None):
        if not isinstance(s, LabelSelector):
            raise SyntaxError('JobSpec: selector: [ {} ] is invalid.'.format(s))
        self._selector = s

    # --------------------------------------------------------------------------------- manualSelector

    # Leave manualSelector unset unless you are certain of what you are doing.

    @property
    def manual_selector(self):
        return self._manual_selector

    @manual_selector.setter
    def manual_selector(self, s=False):
        if not isinstance(s, bool):
            raise SyntaxError('JobSpec: manual_selector: [ {} ] is invalid.'.format(s))
        self._manual_selector = s

    # --------------------------------------------------------------------------------- template

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, t=None):
        if not isinstance(t, PodTemplateSpec):
            raise SyntaxError('JobSpec: template: [ {} ] is invalid.'.format(t))
        self._template = t

    # --------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.parallelism is not None:
            data['parallelism'] = self.parallelism
        if self.completions is not None:
            data['completions'] = self.completions
        if self.active_deadline_seconds is not None:
            data['activeDeadlineSeconds'] = self.active_deadline_seconds
        if self.selector is not None:
            data['selector'] = self.selector.serialize()
        if self.manual_selector is not None:
            data['manualSelector'] = self.manual_selector
        if self.template is not None:
            data['template'] = self.template.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import time

from kubernetes.K8sExceptions import TimedOutException
from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.PersistentVolumeClaim import PersistentVolumeClaim
from kubernetes.models.v1.ResourceRequirements import ResourceRequirements
from kubernetes.models.v1beta1.LabelSelector import LabelSelector
from kubernetes.utils import is_valid_dict

READY_WAIT_TIMEOUT_SECONDS = 60


class K8sPersistentVolumeClaim(K8sObject):

    def __init__(self, config=None, name=None):
        super(K8sPersistentVolumeClaim, self).__init__(
            config=config,
            name=name,
            obj_type='PersistentVolumeClaim'
        )

    # ------------------------------------------------------------------------------------- api calls

    def create(self):
        super(K8sPersistentVolumeClaim, self).create()
        self._wait_for_available()
        return self

    def get(self):
        self.model = PersistentVolumeClaim(self.get_model())
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sPersistentVolumeClaim, self).list(labels=labels)
        claims = list(map(lambda x: PersistentVolumeClaim(x), ls))
        if pattern is not None:
            claims = list(filter(lambda x: pattern in x.name, claims))
        k8s = []
        for x in claims:
            j = K8sPersistentVolumeClaim(config=self.config, name=x.name).from_model(m=x)
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- wait

    def _wait_for_available(self):
        start_time = time.time()
        while not self.model.status.phase == 'Bound':
            time.sleep(0.5)
            self.get()
            self._check_timeout(start_time)

    def _check_timeout(self, start_time=None):
        elapsed_time = time.time() - start_time
        if elapsed_time >= READY_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException(
                "Timed out waiting on readiness of PersistentVolumeClaim: [ {} ]".format(self.name))

    # ------------------------------------------------------------------------------------- accessModes

    @property
    def access_modes(self):
        return self.model.spec.access_modes

    @access_modes.setter
    def access_modes(self, modes=None):
        self.model.spec.access_modes = modes

    # ------------------------------------------------------------------------------------- resources

    @property
    def resources(self):
        return self.model.spec.resources

    @resources.setter
    def resources(self, res=None):
        if not is_valid_dict(res):
            raise SyntaxError(
                'K8sPersistentVolumeClaim: resources: [ {} ] is invalid.'.format(res))

        resources = ResourceRequirements(res)
        self.model.spec.resources = resources

    # ------------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self.model.spec.selector

    @selector.setter
    def selector(self, sel=None):
        if not is_valid_dict(sel):
            raise SyntaxError(
                'K8sPersistentVolumeClaim: selector: [ {} ] is invalid.'.format(sel))

        selector = LabelSelector(sel)
        self.model.spec.selector = selector

    # ------------------------------------------------------------------------------------- storage_class_name

    @property
    def storage_class_name(self):
        return self.model.spec.storage_class_name

    @storage_class_name.setter
    def storage_class_name(self, name=None):
        self.model.spec.storage_class_name = name



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json

import yaml

from kubernetes.models.v1.VolumeMount import VolumeMount


class K8sVolumeMount(object):
    def __init__(self, name=None, mount_path=None, read_only=False, sub_path=None):
        super(K8sVolumeMount, self).__init__()

        self.model = VolumeMount()

        if name is not None:
            self.name = name
        if mount_path is not None:
            self.mount_path = mount_path
        if read_only is not None:
            self.read_only = read_only
        if sub_path is not None:
            self.sub_path = sub_path

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.name

    @name.setter
    def name(self, name=None):
        self.model.name = name

    # ------------------------------------------------------------------------------------- mount_path

    @property
    def mount_path(self):
        return self.model.mount_path

    @mount_path.setter
    def mount_path(self, mp=None):
        self.model.mount_path = mp

    # ------------------------------------------------------------------------------------- read_only

    @property
    def read_only(self):
        return self.model.read_only

    @read_only.setter
    def read_only(self, ro=None):
        self.model.read_only = ro

    # ------------------------------------------------------------------------------------- sub_path

    @property
    def sub_path(self):
        return self.model.sub_path

    @sub_path.setter
    def sub_path(self, sp=None):
        self.model.sub_path = sp

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        return self.model.serialize()

    def as_json(self):
        data = self.serialize()
        dump = json.dumps(data, indent=4)
        return dump

    def as_yaml(self):
        data = self.serialize()
        dump = yaml.dump(data, default_flow_style=False)
        return dump



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sComponentStatus import K8sComponentStatus
from kubernetes.K8sConfig import K8sConfig
from kubernetes.K8sContainer import K8sContainer
from kubernetes.K8sCronJob import K8sCronJob
from kubernetes.K8sDaemonSet import K8sDaemonSet
from kubernetes.K8sDeployment import K8sDeployment
from kubernetes.K8sExceptions import *
from kubernetes.K8sHorizontalPodAutoscaler import K8sHorizontalPodAutoscaler
from kubernetes.K8sJob import K8sJob
from kubernetes.K8sNamespace import K8sNamespace
from kubernetes.K8sNode import K8sNode
from kubernetes.K8sObject import K8sObject
from kubernetes.K8sPersistentVolume import K8sPersistentVolume
from kubernetes.K8sPersistentVolumeClaim import K8sPersistentVolumeClaim
from kubernetes.K8sPetSet import K8sPetSet
from kubernetes.K8sPod import K8sPod
# from kubernetes.K8sReplicaSet import K8sReplicaSet  # should not be used directly
from kubernetes.K8sReplicationController import K8sReplicationController
from kubernetes.K8sSecret import K8sSecret
from kubernetes.K8sService import K8sService
from kubernetes.K8sServiceAccount import K8sServiceAccount
from kubernetes.K8sStatefulSet import K8sStatefulSet
from kubernetes.K8sStorageClass import K8sStorageClass
from kubernetes.K8sVolume import K8sVolume
from kubernetes.K8sVolumeMount import K8sVolumeMount



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import re
from os.path import expanduser, isfile
import logging
import os

import yaml
from yaml import YAMLError

KUBECONFIG_ENV_VAR = "KUBECONFIG"
KUBECONFIG_FILE = "{0}/.kube/config".format(expanduser("~"))
DEFAULT_API_HOST = "http://localhost:8080"
DEFAULT_API_VERSION = "v1"
DEFAULT_NAMESPACE = "default"
SERVICE_ACCOUNT_ROOT = "/var/run/secrets/kubernetes.io/serviceaccount"
SERVICE_ACCOUNT_CA_PATH = "{0}/ca.crt".format(SERVICE_ACCOUNT_ROOT)
SERVICE_ACCOUNT_TOKEN = "{0}/token".format(SERVICE_ACCOUNT_ROOT)
ENV_SERVICE_HOST = "KUBERNETES_SERVICE_HOST"
ENV_SERVICE_PORT = "KUBERNETES_SERVICE_PORT"

VALID_API_VERSIONS = ["v1"]

VALID_IP_RE = re.compile(
    r'^(http[s]?\:\/\/)?((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)){3})(:[0-9]+)?$')
VALID_HOST_RE = re.compile(r'^(http[s]?\:\/\/)?([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-\.]*[A-Za-z])+(:[0-9]+)?$')


class K8sConfig(object):
    def __init__(self, kubeconfig=None, api_host=None, auth=None, cert=None,
                 namespace=None, pull_secret=None, token=None, version=None):
        """
        Pulls configuration from a kubeconfig file, if present, otherwise accepts user-defined parameters.
        See http://kubernetes.io/docs/user-guide/kubeconfig-file/ for information on the kubeconfig file.

        :param kubeconfig: Absolute path to the kubeconfig file, if any.
        :param api_host: Absolute URI where the API server resides.
        :param auth: A tuple of (username, password) for basic authentication.
        :param namespace: The namespace to use. Defaults to 'default'
        :param pull_secret: The password to use when pulling images from the container repository.
        :param token: An authentication token. Mutually exclusive with 'auth'.
        :param version: The version of the API to target. Defaults to 'v1'.
        """

        super(K8sConfig, self).__init__()

        self.api_host = None
        self.auth = None
        self.ca_cert = None
        self.ca_cert_data = None
        self.cert = None
        self.client_certificate = None
        self.client_key = None
        self.pull_secret = None
        self.namespace = None
        self.token = None
        self.version = None

        self._init_with_defaults()

        if kubeconfig is None:
            self._init_with_defaults()
        else:
            self._read_config(filename=kubeconfig)

        # Default fallback host.
        if self.api_host is None:
            logging.debug('Overriding api host with: [ {0} ]'.format(DEFAULT_API_HOST))
            self.api_host = DEFAULT_API_HOST

        # Set defaults if not caught in kubeconfig file or environments.
        if self.namespace is None:
            logging.debug('Overriding namespace with: [ {0} ]'.format(DEFAULT_NAMESPACE))
            self.namespace = DEFAULT_NAMESPACE

        if self.version is None:
            logging.debug('Overriding api version with: [ {0} ]'.format(DEFAULT_API_VERSION))
            self.version = DEFAULT_API_VERSION

        # Process overrides from arguments
        if api_host is not None:
            if not isinstance(api_host, str) or not (VALID_IP_RE.match(api_host) or VALID_HOST_RE.match(api_host)):
                raise SyntaxError('K8sConfig: host: [ {0} ] is invalid.'.format(api_host))
            schema_re = re.compile(r"^http[s]*")
            if not schema_re.search(api_host):
                https_port_re = re.compile(r"\:443$")
                if not https_port_re:
                    logging.debug('Pre-pending http to api host [ {0} ] since port is not 443.'.format(self.api_host))
                    api_host = 'http://{0}'.format(api_host)
                else:
                    logging.debug('Pre-pending https to api host [ {0} ] since port is 443.'.format(self.api_host))
                    api_host = 'https://{0}'.format(api_host)
            self.api_host = api_host

        if auth is not None:
            if not isinstance(auth, tuple):
                raise SyntaxError('K8sConfig: auth: [ {0} ] must be a tuple for basic authentication.'.format(auth))
            self.auth = auth

        if cert is not None:
            if not isinstance(cert, tuple):
                raise SyntaxError('K8sConfig: cert: [ {0} ] must be a tuple for client certificate/key.'.format(cert))
            self.cert = cert

        if namespace is not None:
            if not isinstance(namespace, str):
                raise SyntaxError('K8sConfig: namespace: [ {0} ] must be a string.'.format(namespace))
            self.namespace = namespace

        if pull_secret is not None:
            if not isinstance(pull_secret, list):
                raise SyntaxError('K8sConfig: pull_secret: [ {0} ] must be a list.'.format(pull_secret))
            self.pull_secret = pull_secret

        if token is not None:
            if not isinstance(token, str):
                raise SyntaxError('K8sConfig: token: [ {0} ] must be a string.'.format(token))
            self.token = token

        if version is not None:
            if not isinstance(version, str):
                raise SyntaxError(
                    'K8sConfig: host: [ {0} ] and version: [ {1} ] must be strings.'.format(api_host, version))
            if version not in VALID_API_VERSIONS:
                valid = ", ".join(VALID_API_VERSIONS)
                raise SyntaxError('K8sConfig: api_version: [ {0} ] must be in: [ {1} ]'.format(version, valid))
            self.version = version
        return

    def _init_with_defaults(self):
        # Try to initialize using the environment variable.
        kubeconfig = os.getenv(KUBECONFIG_ENV_VAR, None)
        if kubeconfig is not None:
            self._read_config(filename=kubeconfig)
            return

        # Try to initialize using the ~/.kube/config file.
        if isfile(KUBECONFIG_FILE):
            self._read_config(filename=KUBECONFIG_FILE)
            return

        # Try in-cluster config
        if isfile(SERVICE_ACCOUNT_CA_PATH):
            self._from_cluster()
        return

    def _from_cluster(self):
        # Initialize CA cert.
        if not isfile(SERVICE_ACCOUNT_CA_PATH):
            raise IOError('K8sConfig: Cannot find in-cluster ca certificate [ {0} ] '.format(SERVICE_ACCOUNT_CA_PATH))
        self.ca_cert = SERVICE_ACCOUNT_CA_PATH
        # Initialize the API server host
        host = os.getenv(ENV_SERVICE_HOST, None)
        port = os.getenv(ENV_SERVICE_PORT, None)
        self.api_host = 'https://{0}:{1}'.format(host, port)
        # Initialize the token
        if not isfile(SERVICE_ACCOUNT_TOKEN):
            raise IOError('K8sConfig: Cannot find in-cluster token file [ {1} ]'.format(SERVICE_ACCOUNT_TOKEN))
        with open(SERVICE_ACCOUNT_TOKEN, 'r') as stream:
            self.token = stream.read()
        self.version = DEFAULT_API_VERSION
        return

    def _read_config(self, filename=None):

        if not isfile(filename):
            raise IOError('K8sConfig: kubeconfig: [ {0} ] doesn\'t exist.'.format(filename))
        try:
            with open(filename, 'r') as stream:
                dotconf = yaml.load(stream)
        except YAMLError as err:
            raise SyntaxError('K8sConfig: kubeconfig: [ {0} ] is not a valid YAML file: {1}'.format(filename, err))

        self.clusters = dotconf['clusters']
        self.contexts = dotconf['contexts']
        self.current_context = dotconf['current-context']
        self.current_context_dict = [context['context']
                                     for context in self.contexts
                                     if context['name'] == self.current_context][0]
        self.preferences = dotconf['preferences']
        self.users = dotconf['users']
        self.version = dotconf['apiVersion']

        if self.clusters:
            for cluster in self.clusters:
                if cluster['name'] == self.current_context_dict['cluster']:
                    if 'server' in cluster['cluster']:
                        self.api_host = cluster['cluster']['server']
                    if 'certificate-authority' in cluster['cluster']:
                        self.ca_cert = cluster['cluster']['certificate-authority']
                    if 'certificate-authority-data' in cluster['cluster']:
                        self.ca_cert_data = cluster['cluster']['certificate-authority-data']

        if self.users:
            for user in self.users:
                if user['name'] == self.current_context_dict['user']:
                    if 'username' in user['user'] and 'password' in user['user']:
                        self.auth = (user['user']['username'], user['user']['password'])
                    if 'token' in user['user']:
                        self.token = user['user']['token']
                    if 'client-certificate' in user['user'] and 'client-key' in user['user']:
                        self.client_certificate = user['user']['client-certificate']
                        self.client_key = user['user']['client-key']
                        self.cert = (self.client_certificate, self.client_key)

        if self.contexts:
            for context in self.contexts:
                if context['name'] == self.current_context:
                    if 'namespace' in context['context']:
                        self.namespace = context['context']['namespace']

    def serialize(self):
        data = {}
        if self.api_host is not None:
            data['api_host'] = self.api_host
        if self.auth is not None:
            data['auth'] = self.auth
        if self.cert is not None:
            data['cert'] = self.cert
        if self.namespace is not None:
            data['namespace'] = self.namespace
        if self.pull_secret is not None:
            data['pull_secret'] = self.pull_secret
        if self.token is not None:
            data['token'] = self.token
        if self.version is not None:
            data['version'] = self.version
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_list, filter_model


class PodSecurityContext(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_podsecuritycontext
    """

    def __init__(self, model=None):
        super(PodSecurityContext, self).__init__()

        # TODO(froch): add support for the below
        # self.se_linux_options = None

        self._fs_group = None
        self._run_as_non_root = None
        self._run_as_user = None
        self._supplemental_groups = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'fsGroup' in model:
            self.fs_group = model['fsGroup']
        if 'runAsNonRoot' in model:
            self.run_as_non_root = model['runAsNonRoot']
        if 'runAsUser' in model:
            self.run_as_user = model['runAsUser']
        if 'supplementalGroups' in model:
            self.supplemental_groups = model['supplementalGroups']

    # ------------------------------------------------------------------------------------- fs group

    @property
    def fs_group(self):
        return self._supplemental_groups

    @fs_group.setter
    def fs_group(self, gid=None):
        if not isinstance(gid, int):
            raise SyntaxError('PodSecurityContext: fs_group: [ {0} ] is invalid.'.format(gid))
        self._fs_group = gid

    # ------------------------------------------------------------------------------------- run as non root

    @property
    def run_as_non_root(self):
        return self._run_as_non_root

    @run_as_non_root.setter
    def run_as_non_root(self, b=None):
        if not isinstance(b, bool):
            raise SyntaxError('PodSecurityContext: run_as_non_root: [ {0} ] is invalid.'.format(b))
        self._run_as_non_root = b

    # ------------------------------------------------------------------------------------- run as user

    @property
    def run_as_user(self):
        return self._run_as_user

    @run_as_user.setter
    def run_as_user(self, uid=None):
        if not isinstance(uid, int):
            raise SyntaxError('PodSecurityContext: run_as_user: [ {0} ] is invalid.'.format(uid))
        self._run_as_user = uid

    # ------------------------------------------------------------------------------------- supplemental groups

    @property
    def supplemental_groups(self):
        return self._supplemental_groups

    @supplemental_groups.setter
    def supplemental_groups(self, gids=None):
        if not is_valid_list(gids, int):
            raise SyntaxError('PodSecurityContext: supplemental_groups: [ {0} ] is invalid.'.format(gids))
        self._supplemental_groups = gids

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.fs_group is not None:
            data['fsGroup'] = self.fs_group
        if self.run_as_non_root is not None:
            data['runAsNonRoot'] = self.run_as_non_root
        if self.run_as_user is not None:
            data['runAsUser'] = self.run_as_user
        if self.supplemental_groups is not None:
            data['supplementalGroups'] = self.supplemental_groups
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class ContainerStateWaiting(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_containerstatewaiting
    """

    def __init__(self, model=None):
        super(ContainerStateWaiting, self).__init__()

        self._reason = None
        self._message = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'reason' in model:
            self.reason = model['reason']
        if 'message' in model:
            self.message = model['message']

    # ------------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, msg=None):
        if not is_valid_string(msg):
            raise SyntaxError('ContainerStateWaiting: reason: [ {0} ] is invalid.'.format(msg))
        self._reason = msg

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, msg=None):
        if not is_valid_string(msg):
            raise SyntaxError('ContainerStateWaiting: message: [ {0} ] is invalid.'.format(msg))
        self._message = msg

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.reason is not None:
            data['reason'] = self.reason
        if self.message is not None:
            data['message'] = self.message
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model
from kubernetes.models.v1.KeyToPath import KeyToPath


class ConfigMapVolumeSource(object):
    """
    https://kubernetes.io/docs/api-reference/v1.8/#configmapvolumesource-v1-core

    The contents of the target ConfigMap's Data field will be presented in a volume as files using the keys in the
    Data field as the file names, unless the items element is populated with specific mappings of keys to paths.
    ConfigMap volumes support ownership management and SELinux relabeling.
    """

    def __init__(self, model=None):
        super(ConfigMapVolumeSource, self).__init__()

        self._default_mode = None
        self._items = None
        self._name = None
        self._optional = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'defaultMode' in model:
            self.default_mode = model['defaultMode']
        if 'items' in model:
            self.items = model['items']
        if 'name' in model:
            self.name = model['name']
        if 'optional' in model:
            self.optional = model['optional']

    # ------------------------------------------------------------------------------------- default_mode

    @property
    def default_mode(self):
        return self._default_mode

    @default_mode.setter
    def default_mode(self, mode=None):
        if is_valid_string(mode):
            try:
                mode = int(mode)
            except ValueError:
                raise SyntaxError('ConfigMapVolumeSource: defaultMode: [ {0} ] is invalid.'.format(mode))
        if not isinstance(mode, int):
            raise SyntaxError('ConfigMapVolumeSource: defaultMode: [ {0} ] is invalid.'.format(mode))
        self._default_mode = mode

    # ------------------------------------------------------------------------------------- items

    @property
    def items(self):
        return self._items

    @items.setter
    def items(self, items=None):
        if not isinstance(items, list):
            raise SyntaxError('ConfigMapVolumeSource: items: [ {0} ] is invalid.'.format(items))
        modeled_items = list()
        for i in items:
            tmp_item = KeyToPath(model=i)
            modeled_items.append(tmp_item)
        self._items = modeled_items

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ConfigMapVolumeSource: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- optional

    @property
    def optional(self):
        return self._optional

    @optional.setter
    def optional(self, v=None):
        if not isinstance(v, bool):
            raise SyntaxError('ConfigMapVolumeSource: optional: [ {0} ] is invalid.'.format(v))
        self._optional = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}

        if self.default_mode is not None:
            data['defaultMode '] = self.default_mode
        if self.items is not None:
            tmp_items = list()
            for i in self.items:
                tmp_items.append(i.serialize())
            data['items'] = tmp_items
        if self.name is not None:
            data['name'] = self.name
        if self.optional is not None:
            data['optional'] = self.optional

        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class LoadBalancerIngress(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_loadbalanceringress
    """

    def __init__(self, model=None):
        super(LoadBalancerIngress, self).__init__()

        self._ip = None
        self._hostname = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'ip' in model:
            self.ip = model['ip']
        if 'hostname' in model:
            self.hostname = model['hostname']

    # ------------------------------------------------------------------------------------- ip

    @property
    def ip(self):
        return self._ip

    @ip.setter
    def ip(self, ip=None):
        if not is_valid_string(ip):
            raise SyntaxError('LoadBalancerIngress: ip: [ {} ] is invalid.'.format(ip))
        self._ip = ip

    # ------------------------------------------------------------------------------------- hostname

    @property
    def hostname(self):
        return self._hostname

    @hostname.setter
    def hostname(self, hostname=None):
        if not is_valid_string(hostname):
            raise SyntaxError('LoadBalancerIngress: hostname: [ {} ] is invalid.'.format(hostname))
        self._hostname = hostname

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.ip is not None:
            data['ip'] = self.ip
        if self.hostname is not None:
            data['hostname'] = self.hostname
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import time

from kubernetes import K8sObject
from kubernetes.K8sExceptions import TimedOutException
from kubernetes.models.v1.PersistentVolume import PersistentVolume
from kubernetes.models.v1.PersistentVolumeSpec import PersistentVolumeSpec
from kubernetes.models.v1.Volume import Volume
from kubernetes.utils import is_valid_string

READY_WAIT_TIMEOUT_SECONDS = 60


class K8sPersistentVolume(K8sObject):

    VALID_VOLUME_TYPES = list(filter(
        lambda x: x not in ['emptyDir', 'gitRepo', 'secret'], Volume.VOLUME_TYPES_TO_SOURCE_MAP.keys()))

    def __init__(self, config=None, name=None, type=None):
        super(K8sPersistentVolume, self).__init__(
            config=config,
            name=name,
            obj_type='PersistentVolume')

        self._type = None
        self.model = PersistentVolume()
        self.name = name
        self.type = type

    # ------------------------------------------------------------------------------------- api calls

    def create(self):
        super(K8sPersistentVolume, self).create()
        self._wait_for_available()
        return self

    def get(self):
        self.model = PersistentVolume(self.get_model())
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sPersistentVolume, self).list(labels=labels)
        vols = list(map(lambda x: PersistentVolume(x), ls))
        if pattern is not None:
            vols = list(filter(lambda x: pattern in x.name, vols))
        k8s = []
        for x in vols:
            _types = list(filter(lambda z: z in PersistentVolumeSpec.VOLUME_TYPES_TO_SOURCE_MAP, dir(x.spec)))
            j = K8sPersistentVolume(config=self.config, name=self.name, type=_types[0]).from_model(m=x)
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- wait

    def _wait_for_available(self):
        start_time = time.time()
        while not self.model.status.phase == 'Available':
            time.sleep(0.5)
            self.get()
            self._check_timeout(start_time)

    def _check_timeout(self, start_time=None):
        elapsed_time = time.time() - start_time
        if elapsed_time >= READY_WAIT_TIMEOUT_SECONDS:  # timeout
            raise TimedOutException(
                "Timed out waiting on readiness of PersistentVolume: [ {} ]".format(self.name))

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.metadata.name

    @name.setter
    def name(self, name=None):
        self.model.metadata.name = name
        self.model.metadata.labels['name'] = name

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, t=None):
        if t not in self.VALID_VOLUME_TYPES:
            raise SyntaxError(
                'K8sPersistentVolume: type: [ {} ] is invalid.'.format(t))

        self._type = t
        setattr(self.model.spec, t, Volume.vol_type_to_source(t))

    # ------------------------------------------------------------------------------------- source

    @property
    def source(self):
        return getattr(self.model.spec, self._type, None)

    @source.setter
    def source(self, s=None):
        raise NotImplementedError()

    # ------------------------------------------------------------------------------------- accessModes

    @property
    def access_modes(self):
        return self.model.spec.access_modes

    @access_modes.setter
    def access_modes(self, modes=None):
        self.model.spec.access_modes = modes

    # ------------------------------------------------------------------------------------- capacity

    @property
    def capacity(self):
        return self.model.spec.capacity

    @capacity.setter
    def capacity(self, cap=None):
        self.model.spec.capacity = cap

    # ------------------------------------------------------------------------------------- path (hostPath)

    @property
    def path(self):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        return self.source.path

    @path.setter
    def path(self, p=None):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        self.source.path = p

    # ------------------------------------------------------------------------------------- volume_id (AWS)

    @property
    def volume_id(self):
        if not hasattr(self.source, 'volume_id'):
            raise NotImplementedError()
        return self.source.volume_id

    @volume_id.setter
    def volume_id(self, vid=None):
        if not hasattr(self.source, 'volume_id'):
            raise NotImplementedError()
        self.source.volume_id = vid

    # ------------------------------------------------------------------------------------- pd_name (GCE)

    @property
    def pd_name(self):
        if not hasattr(self.source, 'pd_name'):
            raise NotImplementedError()
        return self.source.pd_name

    @pd_name.setter
    def pd_name(self, pd=None):
        if not hasattr(self.source, 'pd_name'):
            raise NotImplementedError()
        self.source.pd_name = pd

    # ------------------------------------------------------------------------------------- fs_type (AWS, GCE)

    @property
    def fs_type(self):
        if not hasattr(self.source, 'fs_type'):
            raise NotImplementedError()
        return self.source.fs_type

    @fs_type.setter
    def fs_type(self, t=None):
        if not hasattr(self.source, 'fs_type'):
            raise NotImplementedError()
        self.source.fs_type = t

    # ------------------------------------------------------------------------------------- nfs_server

    @property
    def nfs_server(self):
        if not hasattr(self.source, 'server'):
            raise NotImplementedError()
        return self.source.server

    @nfs_server.setter
    def nfs_server(self, s=None):
        if not hasattr(self.source, 'server'):
            raise NotImplementedError()
        self.source.server = s

    # ------------------------------------------------------------------------------------- nfs_path

    @property
    def nfs_path(self):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        return self.source.path

    @nfs_path.setter
    def nfs_path(self, p=None):
        if not hasattr(self.source, 'path'):
            raise NotImplementedError()
        self.source.path = p

    # ------------------------------------------------------------------------------------- storage_class_name

    @property
    def storage_class_name(self):
        return self.model.spec.storage_class_name

    @storage_class_name.setter
    def storage_class_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError("K8sPersistentVolume: storage_class_name: [ {} ] is invalid.".format(name))
        self.model.spec.storage_class_name = name

    # ------------------------------------------------------------------------------------- reclaim_policy

    @property
    def reclaim_policy(self):
        return self.model.spec.reclaim_policy

    @reclaim_policy.setter
    def reclaim_policy(self, pol=None):
        self.model.spec.reclaim_policy = pol



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import filter_model


class EmptyDirVolumeSource(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_emptydirvolumesource
    """

    VALID_MEDIA = ['', 'Memory']

    def __init__(self, model=None):
        super(EmptyDirVolumeSource, self).__init__()

        self._medium = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'medium' in model:
            self.medium = model['medium']

    # ------------------------------------------------------------------------------------- medium

    @property
    def medium(self):
        return self._medium

    @medium.setter
    def medium(self, medium=None):
        if medium is None or medium not in EmptyDirVolumeSource.VALID_MEDIA:
            raise SyntaxError('EmptyDirVolumeSource: medium: [ {0} ] is invalid.'.format(medium))
        self._medium = medium

    # ------------------------------------------------------------------------------------- medium

    def serialize(self):
        data = {}
        if self.medium is not None:
            data['medium'] = self.medium
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import json

import yaml

from kubernetes.K8sVolumeMount import K8sVolumeMount
from kubernetes.models.v1.Capabilities import Capabilities
from kubernetes.models.v1.Container import Container
from kubernetes.models.v1.ContainerPort import ContainerPort
from kubernetes.models.v1.EnvVar import EnvVar
from kubernetes.models.v1.Probe import Probe
from kubernetes.models.v1.ResourceRequirements import ResourceRequirements


class K8sContainer(object):
    """
    The K8sContainer object currently supports the default Kubernetes container runtime, ie. Docker.
    """

    def __init__(self, name=None, image=None):
        super(K8sContainer, self).__init__()

        self.model = Container()
        self.name = name
        self.image = image

    def __eq__(self, other):
        if isinstance(other, self.__class__):
            return self.model.name == other.model.name and self.model.image == other.model.image
        return NotImplemented

    # -------------------------------------------------------------------------------------  add

    def add_port(self, container_port=None, host_port=None, name=None, protocol=None, host_ip=None):
        p = ContainerPort()
        if container_port is not None:
            p.container_port = container_port
        if host_port is not None:
            p.host_port = host_port
        if name is not None:
            p.name = name
        if protocol is not None:
            p.protocol = protocol
        if host_ip is not None:
            p.host_ip = host_ip

        ports = self.model.ports
        if ports is None:
            ports = []
        ports.append(p)

        self.model.ports = ports

    def add_env(self, name=None, value=None):
        e = {'name': name}

        if isinstance(value, dict):
            if 'valueFrom' in value and len(value) == 1:
                e['valueFrom'] = value['valueFrom']
        elif isinstance(value, str):
            e['value'] = value
        else:
            raise SyntaxError('K8sContainer.add_env() value: [ {} ] is invalid.')

        env_var = EnvVar(e)
        env = self.model.env
        if env is None:
            env = []
        env.append(env_var)
        self.model.env = env

    def add_volume_mount(self, mount=None):
        if not isinstance(mount, K8sVolumeMount):
            raise SyntaxError('K8sContainer.add_volume_mount() mount: [ {} ] is invalid.'.format(mount))
        mounts = self.model.volume_mounts
        if mount.model not in mounts:
            mounts.append(mount.model)
        self.model.volume_mounts = mounts

    def add_liveness_probe(self, **kwargs):
        if not isinstance(kwargs, dict):
            raise SyntaxError('K8sContainer: could not add liveness_probe: [ {} ]'.format(kwargs))
        probe = Probe(kwargs)
        self.liveness_probe = probe

    def add_readiness_probe(self, **kwargs):
        if not isinstance(kwargs, dict):
            raise SyntaxError('K8sContainer: could not add readiness_probe: [ {} ]'.format(kwargs))
        probe = Probe(kwargs)
        self.readiness_probe = probe

    def add_capabilities(self, c=None):
        cap = Capabilities({'add': c})
        self.capabilities = cap

    def drop_capabilities(self, c=None):
        cap = Capabilities({'drop': c})
        self.capabilities = cap

    # -------------------------------------------------------------------------------------  args

    @property
    def args(self):
        return self.model.args

    @args.setter
    def args(self, args=None):
        self.model.args = args

    # -------------------------------------------------------------------------------------  command

    @property
    def command(self):
        return self.model.command

    @command.setter
    def command(self, cmd=None):
        self.model.command = cmd

    # -------------------------------------------------------------------------------------  env

    @property
    def env(self):
        env = []
        for x in self.model.env:
            env.append(x)
        return env

    @env.setter
    def env(self, env=None):
        raise NotImplementedError()

    # -------------------------------------------------------------------------------------  ports

    @property
    def ports(self):
        return self.model.ports

    @ports.setter
    def ports(self, ports=None):
        self.model.ports = ports

    # -------------------------------------------------------------------------------------  livenessProbe

    @property
    def liveness_probe(self):
        return self.model.liveness_probe

    @liveness_probe.setter
    def liveness_probe(self, probe=None):
        self.model.liveness_probe = probe

    # -------------------------------------------------------------------------------------  readinessProbe

    @property
    def readiness_probe(self):
        return self.model.readiness_probe

    @readiness_probe.setter
    def readiness_probe(self, probe=None):
        self.model.readiness_probe = probe

    # -------------------------------------------------------------------------------------  resources

    @property
    def resources(self):
        return self.model.resources

    @resources.setter
    def resources(self, res=None):
        r = ResourceRequirements(res)
        self.model.resources = r

    # -------------------------------------------------------------------------------------  name

    @property
    def name(self):
        return self.model.name

    @name.setter
    def name(self, name=None):
        self.model.name = name

    # -------------------------------------------------------------------------------------  image

    @property
    def image(self):
        return self.model.image

    @image.setter
    def image(self, image=None):
        self.model.image = image

    # -------------------------------------------------------------------------------------  image_pull_policy

    @property
    def image_pull_policy(self):
        return self.model.image_pull_policy

    @image_pull_policy.setter
    def image_pull_policy(self, pull_policy=None):
        self.model.image_pull_policy = pull_policy

    # -------------------------------------------------------------------------------------  volume_mounts

    @property
    def volume_mounts(self):
        return self.model.volume_mounts

    @volume_mounts.setter
    def volume_mounts(self, mounts=None):
        self.model.volume_mounts = mounts

    # -------------------------------------------------------------------------------------  capabilities

    @property
    def capabilities(self):
        return self.model.security_context.capabilities

    @capabilities.setter
    def capabilities(self, c=None):
        self.model.security_context.capabilities = c

    # -------------------------------------------------------------------------------------  seLinuxOptions

    @property
    def se_linux_options(self):
        return self.model.security_context.se_linux_options

    @se_linux_options.setter
    def se_linux_options(self, o=None):
        self.model.security_context.se_linux_options = o

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        return self.model.serialize()

    def as_json(self):
        data = self.serialize()
        dump = json.dumps(data, indent=4)
        return dump

    def as_yaml(self):
        data = self.serialize()
        dump = yaml.dump(data, default_flow_style=False)
        return dump



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.ObjectMeta import ObjectMeta
from kubernetes.models.v1.ServiceSpec import ServiceSpec
from kubernetes.models.v1.ServiceStatus import ServiceStatus
from kubernetes.models.v1.ServicePort import ServicePort
from kubernetes.utils import is_valid_dict, is_valid_string


class Service(BaseModel):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_service
    """

    def __init__(self, model=None):
        super(Service, self).__init__()

        self.kind = 'Service'
        self.api_version = 'v1'

        self.spec = ServiceSpec()
        self.status = ServiceStatus()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(Service, self).build_with_model(model)

        if 'spec' in model:
            self.spec = ServiceSpec(model['spec'])
        if 'status' in model:
            self.status = ServiceStatus(model['status'])

    # ------------------------------------------------------------------------------------- add

    def add_annotation(self, k=None, v=None):
        anns = self.metadata.annotations
        if anns is None:
            anns = {}
        anns.update({k: v})
        self.metadata.annotations = anns
        return self

    def add_label(self, k=None, v=None):
        labels = self.metadata.labels
        if labels is None:
            labels = {}
        labels.update({k: v})
        self.metadata.labels = labels
        return self

    def add_port(self, name=None, port=None, target_port=None, protocol=None, node_port=None):
        if not is_valid_string(name):
            raise SyntaxError('Service.add_port() name: [ {} ] is invalid.'.format(name))
        if port is None or not (isinstance(port, str) or isinstance(port, int)):
            raise SyntaxError('Service.add_port() port: [ {} ] is invalid.'.format(name))
        ports = []
        # exists previously
        for p in self.spec.ports:
            if int(p.port) == int(port):
                p.name = name
                p.target_port = target_port
                p.protocol = protocol
                p.node_port = node_port
            ports.append(p)
        # doesn't exist yet
        if int(port) not in [int(x.port) for x in ports]:
            p = ServicePort()
            p.name = name
            p.port = port
            p.target_port = target_port
            p.protocol = protocol
            p.node_port = node_port
            ports.append(p)
        self.spec.ports = ports

    def add_selector(self, selector=None):
        if not is_valid_dict(selector):
            raise SyntaxError('Service.add_selector() selector: [ {} ] is invalid.'.format(selector))
        s = self.spec.selector
        if s is None:
            s = {}
        s.update(selector)
        self.spec.selector = s

    # ------------------------------------------------------------------------------------- spec

    @property
    def spec(self):
        return self._spec

    @spec.setter
    def spec(self, spec=None):
        if not isinstance(spec, ServiceSpec):
            raise SyntaxError('Service: spec: [ {0} ] is invalid.'.format(spec))
        self._spec = spec

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status=None):
        if not isinstance(status, ServiceStatus):
            raise SyntaxError('Service: status: [ {0} ] is invalid.'.format(status))
        self._status = status

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(Service, self).serialize()

        if self.spec is not None:
            data['spec'] = self.spec.serialize()
        if self.status is not None:
            data['status'] = self.status.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.ListMeta import ListMeta
from kubernetes.models.unversioned.ListModel import ListModel
from kubernetes.models.v1.Node import Node
from kubernetes.utils import filter_model, is_valid_list


class NodeList(ListModel):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_nodelist
    """

    def __init__(self, model=None):
        super(NodeList, self).__init__()

        self.kind = 'NodeList'
        self.api_version = 'v1'

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'kind' in model:
            self.kind = model['kind']
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'metadata' in model:
            self.metadata = ListMeta(model['metadata'])
        if 'items' in model:
            if isinstance(model['items'], list):
                l = list()
                for i in model['items']:
                    l.append(Node(model=i))
                self.items = l
            else:
                raise SyntaxError('NodeList: items: [ {0} ] is invalid.'.format(model['items']))

    # ------------------------------------------------------------------------------------- items

    @property
    def items(self):
        return self._items

    @items.setter
    def items(self, i=None):
        if not is_valid_list(target=i, element_class=Node):
            raise SyntaxError('NodeList: items: [ {0} ] is invalid.'.format(i))
        self._items = i

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(NodeList, self).serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PersistentVolumeClaim import PersistentVolumeClaim
from kubernetes.models.v1.PodTemplateSpec import PodTemplateSpec
from kubernetes.models.v1beta1.LabelSelector import LabelSelector
from kubernetes.utils import is_valid_list, is_valid_string


class StatefulSetSpec(object):

    def __init__(self, model=None):
        super(StatefulSetSpec, self).__init__()

        self._replicas = None
        self._selector = None
        self._template = None
        self._volume_claim_templates = None
        self._service_name = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'replicas' in model:
            self.replicas = model['replicas']
        if 'selector' in model:
            self.selector = LabelSelector(model['selector'])
        if 'template' in model:
            self.template = PodTemplateSpec(model['template'])
        if 'volumeClaimTemplates' in model:
            vcts = [PersistentVolumeClaim(x) for x in model['volumeClaimTemplates']]
            self.volume_claim_templates = vcts
        if 'serviceName' in model:
            self.service_name = model['serviceName']

    # ------------------------------------------------------------------------------------- replicas

    @property
    def replicas(self):
        return self._replicas

    @replicas.setter
    def replicas(self, r=None):
        if not isinstance(r, int):
            raise SyntaxError('StatefulSetSpec: replicas: [ {} ] is invalid.'.format(r))
        self._replicas = r

    # ------------------------------------------------------------------------------------- selector

    @property
    def selector(self):
        return self._selector

    @selector.setter
    def selector(self, s=None):
        if not isinstance(s, LabelSelector):
            raise SyntaxError('StatefulSetSpec: selector: [ {} ] is invalid.'.format(s))
        self._selector = s

    # ------------------------------------------------------------------------------------- template

    @property
    def template(self):
        return self._template

    @template.setter
    def template(self, t=None):
        if not isinstance(t, PodTemplateSpec):
            raise SyntaxError('StatefulSetSpec: template: [ {} ] is invalid.'.format(t))
        self._template = t

    # ------------------------------------------------------------------------------------- volumeClaimTemplates

    @property
    def volume_claim_templates(self):
        return self._volume_claim_templates

    @volume_claim_templates.setter
    def volume_claim_templates(self, vcts=None):
        if not is_valid_list(vcts, PersistentVolumeClaim):
            raise SyntaxError('StatefulSetSpec: volume_claim_templates: [ {} ] is invalid.'.format(vcts))
        self._volume_claim_templates = vcts

    # ------------------------------------------------------------------------------------- serviceName

    @property
    def service_name(self):
        return self._service_name

    @service_name.setter
    def service_name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('StatefulSetSpec: service_name: [ {} ] is invalid.'.format(name))
        self._service_name = name

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.replicas is not None:
            data['replicas'] = self.replicas
        if self.selector is not None:
            data['selector'] = self.selector
        if self.template is not None:
            data['template'] = self.template.serialize()
        if self.volume_claim_templates is not None:
            data['volumeClaimTemplates'] = [x.serialize() for x in self.volume_claim_templates]
        if self.service_name is not None:
            data['serviceName'] = self.service_name
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1beta1.StorageClass import StorageClass


class K8sStorageClass(K8sObject):
    def __init__(self, config=None, name=None):

        super(K8sStorageClass, self).__init__(
            config=config,
            name=name,
            obj_type='StorageClass'
        )

    # -------------------------------------------------------------------------------------  override

    def get(self):
        self.model = StorageClass(self.get_model())
        return self

    def create(self):
        super(K8sStorageClass, self).create()
        self.get()
        return self

    def update(self):
        super(K8sStorageClass, self).update()
        self.get()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sStorageClass, self).list(labels=labels)
        sclasses = list(map(lambda x: StorageClass(x), ls))
        if pattern is not None:
            sclasses = list(filter(lambda x: pattern in x.name, sclasses))
        k8s = []
        for x in sclasses:
            j = K8sStorageClass(config=self.config, name=x.name).from_model(m=x)
            k8s.append(j)
        return k8s



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.ListMeta import ListMeta
from kubernetes.utils import is_valid_string


class ListModel(object):
    def __init__(self):
        super(ListModel, self).__init__()

        self._api_version = None
        self._kind = None
        self._metadata = ListMeta()
        self._items = None

    def build_with_model(self, model=None):
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'kind' in model:
            self.kind = model['kind']
        if 'metadata' in model:
            self.metadata = ListMeta(model=model['metadata'])

    # ------------------------------------------------------------------------------------- apiVersion

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('ListModel: api_version: [ {} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('ListModel: kind: [ {} ] is invalid.'.format(k))
        self._kind = k

    # ------------------------------------------------------------------------------------- metadata

    @property
    def metadata(self):
        return self._metadata

    @metadata.setter
    def metadata(self, meta=None):
        if not isinstance(meta, ListMeta):
            raise SyntaxError('ListModel: metadata: [ {} ] is invalid.'.format(meta))
        self._metadata = meta

    # ------------------------------------------------------------------------------------- items

    @property
    def items(self):
        return self._items

    @items.setter
    def items(self, l=None):
        if not isinstance(l, list):
            raise SyntaxError('ListModel: items: [ {} ] is invalid.'.format(l))
        self._items = l

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.api_version is not None:
            data['apiVersion'] = self.api_version
        if self.kind is not None:
            data['kind'] = self.kind
        if self.metadata is not None:
            data['metadata'] = self.metadata.serialize()
        if self.items is not None:
            l = list()
            for i in self.items:
                l.append(i.serialize())
            data['items'] = l
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import filter_model, is_valid_string


class OwnerReference(object):

    def __init__(self, model=None):
        super(OwnerReference, self).__init__()

        self._api_version = None
        self._kind = None
        self._name = None
        self._uid = None
        self._controller = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'apiVersion' in model:
            self.api_version = model['apiVersion']
        if 'kind' in model:
            self.kind = model['kind']
        if 'name' in model:
            self.model = model['name']
        if 'uid' in model:
            self.uid = model['uid']
        if 'controller' in model:
            self.controller = model['controller']

    # ------------------------------------------------------------------------------------- api version

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('OwnerReference: api_version: [ {0} ] is invalid.'.format(v))
        self._api_version = v

    # ------------------------------------------------------------------------------------- kind

    @property
    def kind(self):
        return self._kind

    @kind.setter
    def kind(self, kind=None):
        if not is_valid_string(kind):
            raise SyntaxError('OwnerReference: kind: [ {0} ] is invalid.'.format(kind))
        self._kind = kind

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('OwnerReference: name: [ {0} ] is invalid.'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- uid

    @property
    def uid(self):
        return self._uid

    @uid.setter
    def uid(self, uid=None):
        if not is_valid_string(uid):
            raise SyntaxError('OwnerReference: uid: [ {0} ] is invalid.'.format(uid))
        self._uid = uid

    # ------------------------------------------------------------------------------------- controller

    @property
    def controller(self):
        return self._controller

    @controller.setter
    def controller(self, c=None):
        if not isinstance(c, bool):
            raise SyntaxError('OwnerReference: controller: [ {0} ] is invalid.'.format(c))
        self._controller = c

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.api_version:
            data['apiVersion'] = self.api_version
        if self.kind:
            data['kind'] = self.kind
        if self.name:
            data['name'] = self.name
        if self.uid:
            data['uid'] = self.uid
        if self.controller:
            data['controller'] = self.controller
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.utils import filter_model


class AttachedVolume(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_attachedvolume
    """

    def __init__(self, model=None):
        super(AttachedVolume, self).__init__()

        self._name = None
        self._device_path = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']
        if 'devicePath' in model:
            self.device_path = model['devicePath']

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, v):
        if not isinstance(v, str):
            raise SyntaxError('AttachedVolume: name: [ {0} ] is invalid.'.format(v))
        self._name = v

    # ------------------------------------------------------------------------------------- device_path

    @property
    def device_path(self):
        return self._device_path

    @device_path.setter
    def device_path(self, v):
        if not isinstance(v, str):
            raise SyntaxError('AttachedVolume: device_path: [ {0} ] is invalid.'.format(v))
        self._device_path = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name:
            data['name'] = self.name
        if self.device_path:
            data['devicePath'] = self.device_path
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#
from kubernetes.utils import filter_model


class ComponentCondition(object):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_componentcondition
    """

    VALID_TYPE = ['Healthy']
    VALID_STATUS = ['True', 'False', 'Unknown']

    def __init__(self, model=None):
        super(ComponentCondition, self).__init__()

        self._type = None
        self._status = None
        self._message = None
        self._error = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'type' in model:
            self.type = model['type']
        if 'status' in model:
            self.status = model['status']
        if 'message' in model:
            self.message = model['message']
        if 'error' in model:
            self.error = model['error']

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, v):
        if not isinstance(v, str) or v not in self.VALID_TYPE:
            raise SyntaxError('ComponentCondition: type: [ {0} ] is invalid.'.format(v))
        self._type = v

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, v):
        if not isinstance(v, str) or v not in self.VALID_STATUS:
            raise SyntaxError('ComponentCondition: status: [ {0} ] is invalid.'.format(v))
        self._status = v

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, v):
        if not isinstance(v, str):
            raise SyntaxError('ComponentCondition: message: [ {0} ] is invalid.'.format(v))
        self._message = v

    # ------------------------------------------------------------------------------------- error

    @property
    def error(self):
        return self._error

    @error.setter
    def error(self, v):
        if not isinstance(v, str):
            raise SyntaxError('ComponentCondition: error: [ {0} ] is invalid.'.format(v))
        self._error = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.type:
            data['type'] = self.type
        if self.type:
            data['status'] = self.status
        if self.type:
            data['message'] = self.message
        if self.type:
            data['error'] = self.error
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class Toleration(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#toleration-v1-core
    """

    def __init__(self, model=None):
        super(Toleration, self).__init__()

        self._effect = None
        self._key = None
        self._operator = None
        self._toleration_seconds = None
        self._value = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'effect' in model:
            self.effect = model['effect']
        if 'key' in model:
            self.key = model['key']
        if 'operator' in model:
            self.operator = model['operator']
        if 'tolerationSeconds' in model:
            self.toleration_seconds = model['tolerationSeconds']
        if 'value' in model:
            self.value = model['value']

    # ------------------------------------------------------------------------------------- effect

    @property
    def effect(self):
        return self._effect

    @effect.setter
    def effect(self, e=None):
        if not is_valid_string(e):
            raise SyntaxError('Toleration: effect: [ {} ] is invalid'.format(e))
        self._effect = e

    # ------------------------------------------------------------------------------------- key

    @property
    def key(self):
        return self._key

    @key.setter
    def key(self, k=None):
        if not is_valid_string(k):
            raise SyntaxError('Toleration: effect: [ {} ] is invalid.'.format(k))
        self._key = k

    # ------------------------------------------------------------------------------------- operator

    @property
    def operator(self):
        return self._operator

    @operator.setter
    def operator(self, o=None):
        if not is_valid_string(o):
            raise SyntaxError('Toleration: operator: [ {} ] is invalid.'.format(o))
        self._operator = o

    # ------------------------------------------------------------------------------------- tolerationSeconds

    @property
    def toleration_seconds(self):
        return self._toleration_seconds

    @toleration_seconds.setter
    def toleration_seconds(self, ts=None):
        if not isinstance(ts, int):
            raise SyntaxError('Toleration: toleration_seconds: [ {} ] is invalid.'.format(ts))
        self._toleration_seconds = ts

    # ------------------------------------------------------------------------------------- value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, v=None):
        if not is_valid_string(v):
            raise SyntaxError('Toleration: value: [ {} ] is invalid.'.format(v))
        self._value = v

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.effect is not None:
            data['effect'] = self.effect
        if self.key is not None:
            data['key'] = self.key
        if self.operator is not None:
            data['operator'] = self.operator
        if self.toleration_seconds is not None:
            data['tolerationSeconds'] = self.toleration_seconds
        if self.value is not None:
            data['value'] = self.value
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#


class InvalidObjectException(Exception):
    def __init__(self, *args, **kwargs):
        super(InvalidObjectException, self).__init__(*args, **kwargs)


class UnauthorizedException(Exception):
    def __init__(self, *args, **kwargs):
        super(UnauthorizedException, self).__init__(*args, **kwargs)


class NotFoundException(Exception):
    def __init__(self, *args, **kwargs):
        super(NotFoundException, self).__init__(*args, **kwargs)


class UnprocessableEntityException(Exception):
    def __init__(self, *args, **kwargs):
        super(UnprocessableEntityException, self).__init__(*args, **kwargs)


class BadRequestException(Exception):
    def __init__(self, *args, **kwargs):
        super(BadRequestException, self).__init__(*args, **kwargs)


class AlreadyExistsException(Exception):
    def __init__(self, *args, **kwargs):
        super(AlreadyExistsException, self).__init__(*args, **kwargs)


class TimedOutException(Exception):
    def __init__(self, *args, **kwargs):
        super(TimedOutException, self).__init__(*args, **kwargs)


class PodNotReadyException(Exception):
    def __init__(self, *args, **kwargs):
        super(PodNotReadyException, self).__init__(*args, **kwargs)


class VersionMismatchException(Exception):
    def __init__(self, *args, **kwargs):
        super(VersionMismatchException, self).__init__(*args, **kwargs)


class DrainNodeException(Exception):
    def __init__(self, *args, **kwargs):
        super(DrainNodeException, self).__init__(*args, **kwargs)


class CronJobAlreadyRunningException(Exception):
    def __init__(self, *args, **kwargs):
        super(CronJobAlreadyRunningException, self).__init__(*args, **kwargs)


class CronJobRunException(Exception):
    def __init__(self, *args, **kwargs):
        super(CronJobRunException, self).__init__(*args, **kwargs)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.ObjectMeta import ObjectMeta
from kubernetes.models.v1.NamespaceSpec import NamespaceSpec
from kubernetes.models.v1.NamespaceStatus import NamespaceStatus


class Namespace(BaseModel):
    """
    https://kubernetes.io/docs/api-reference/v1/definitions/#_v1_namespace
    """

    def __init__(self, model=None):
        super(Namespace, self).__init__()

        self.kind = 'Namespace'
        self.api_version = 'v1'
        self.spec = NamespaceSpec()
        self.status = NamespaceStatus()

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(Namespace, self).build_with_model(model)

        if 'spec' in model:
            self.spec = NamespaceSpec(model['spec'])
        if 'status' in model:
            self.status = NamespaceStatus(model['status'])

    # --------------------------------------------------------------------------------- spec

    @property
    def spec(self):
        return self._spec

    @spec.setter
    def spec(self, s=None):
        if not isinstance(s, NamespaceSpec):
            raise SyntaxError('Namespace: spec: [ {} ] is invalid.'.format(s))
        self._spec = s

    # --------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, s=None):
        if not isinstance(s, NamespaceStatus):
            raise SyntaxError('Job: status: [ {} ] is invalid.'.format(s))
        self._status = s

    # --------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(Namespace, self).serialize()
        if self.spec is not None:
            data['spec'] = self.spec.serialize()
        if self.status is not None:
            data['status'] = self.status.serialize()
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.ConfigMap import ConfigMap


class K8sConfigMap(K8sObject):

    def __init__(self, config=None, name=None):

        _type = 'ConfigMap'

        super(K8sConfigMap, self).__init__(
            config=config,
            obj_type=_type,
            name=name
        )

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sConfigMap, self).create()
        self.get()
        return self

    def update(self):
        super(K8sConfigMap, self).update()
        self.get()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sConfigMap, self).list(labels=labels)
        cm = list(map(lambda x: ConfigMap(x), ls))
        if pattern is not None:
            cm = list(filter(lambda x: pattern in x.name, cm))
        k8s = []
        for x in cm:
            j = K8sConfigMap(config=self.config, name=x.name).from_model(m=x)
            k8s.append(j)
        return k8s

    # -------------------------------------------------------------------------------------  get

    def get(self):
        self.model = ConfigMap(self.get_model())
        return self

    # -------------------------------------------------------------------------------------  add

    def add_data(self, k=None, v=None):
        if k is None or v is None:
            raise SyntaxError(
                'K8sConfigMap.add_data(): Key [ {0} ] or Value [ {1} ] is invalid.'.format(k, v))

        tmp_data = self.data
        if tmp_data is None:
            tmp_data = dict()
        if k in tmp_data.keys():
            tmp_data[k] = v
        else:
            tmp_data.update({k: v})
        self.data = tmp_data
        return self

    # -------------------------------------------------------------------------------------  data

    @property
    def data(self):
        return self.model.data

    @data.setter
    def data(self, v=None):
        self.model.data = v



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string


class PersistentVolumeStatus(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_persistentvolumestatus
    """

    def __init__(self, model=None):
        super(PersistentVolumeStatus, self).__init__()

        self._phase = None
        self._message = None
        self._reason = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'phase' in model:
            self.phase = model['phase']
        if 'message' in model:
            self.message = model['message']
        if 'reason' in model:
            self.reason = model['reason']

    # ------------------------------------------------------------------------------------- phase

    @property
    def phase(self):
        return self._phase

    @phase.setter
    def phase(self, p=None):
        if not is_valid_string(p):
            raise SyntaxError('PersistentVolumeStatus: phase: [ {} ] is invalid.'.format(p))
        self._phase = p

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._phase

    @message.setter
    def message(self, m=None):
        if not is_valid_string(m):
            raise SyntaxError('PersistentVolumeStatus: message: [ {} ] is invalid.'.format(m))
        self._message = m

    # ------------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, r=None):
        if not is_valid_string(r):
            raise SyntaxError('PersistentVolumeStatus: reason: [ {} ] is invalid.'.format(r))
        self._reason = r

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.phase is not None:
            data['phase'] = self.phase
        if self.message is not None:
            data['message'] = self.message
        if self.reason is not None:
            data['reason'] = self.reason
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.unversioned.BaseModel import BaseModel
from kubernetes.models.v1.EventSource import EventSource
from kubernetes.models.v1.ObjectReference import ObjectReference
from kubernetes.utils import is_valid_date_time, is_valid_string


class Event(BaseModel):
    """
    https://kubernetes.io/docs/api-reference/v1.5/#event-v1
    """

    def __init__(self, model=None):
        super(Event, self).__init__(model=model)

        self._count = None
        self._first_timestamp = None
        self._involved_object = ObjectReference()
        self._last_timestamp = None
        self._message = None
        self._reason = None
        self._source = EventSource()
        self._type = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        super(Event, self).build_with_model(model)

        if 'count' in model:
            self.count = model['count']
        if 'firstTimestamp' in model:
            self.first_timestamp = model['firstTimestamp']
        if 'involvedObject' in model:
            self.involved_object = ObjectReference(model['involvedObject'])
        if 'lastTimestamp' in model:
            self.last_timestamp = model['lastTimestamp']
        if 'message' in model:
            self.message = model['message']
        if 'reason' in model:
            self.reason = model['reason']
        if 'source' in model:
            self.source = EventSource(model['source'])
        if 'type' in model:
            self.type = model['type']

    # ------------------------------------------------------------------------------------- count

    @property
    def count(self):
        return self._count

    @count.setter
    def count(self, c=None):
        if not isinstance(c, int):
            raise SyntaxError("Event: count: [ {} ] is invalid.".format(c))
        self._count = c

    # ------------------------------------------------------------------------------------- firstTimestamp

    @property
    def first_timestamp(self):
        return self._first_timestamp

    @first_timestamp.setter
    def first_timestamp(self, t=None):
        if not is_valid_date_time(t):
            raise SyntaxError("Event: first_timestamp: [ {} ] is invalid.".format(t))
        self._first_timestamp = t

    # ------------------------------------------------------------------------------------- involvedObject

    @property
    def involved_object(self):
        return self._involved_object

    @involved_object.setter
    def involved_object(self, o=None):
        if not isinstance(o, ObjectReference):
            raise SyntaxError("Event: involved_object: [ {} ] is invalid.".format(o))
        self._involved_object = o

    # ------------------------------------------------------------------------------------- lastTimestamp

    @property
    def last_timestamp(self):
        return self._last_timestamp

    @last_timestamp.setter
    def last_timestamp(self, t=None):
        if not is_valid_date_time(t):
            raise SyntaxError("Event: last_timestamp: [ {} ] is invalid.".format(t))
        self._last_timestamp = t

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, m=None):
        if not is_valid_string(m):
            raise SyntaxError("Event: message: [ {} ] is invalid.".format(m))
        self._message = m

    # ------------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, r=None):
        if not is_valid_string(r):
            raise SyntaxError("Event: reason: [ {} ] is invalid.".format(r))
        self._reason = r

    # ------------------------------------------------------------------------------------- source

    @property
    def source(self):
        return self._source

    @source.setter
    def source(self, s=None):
        if not isinstance(s, EventSource):
            raise SyntaxError("Event: source: [ {} ] is invalid.".format(s))
        self._source = s

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError("Event: type: [ {} ] is invalid.".format(t))
        self._type = t

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = super(Event, self).serialize()

        if self.count is not None:
            data['count'] = self.count
        if self.first_timestamp is not None:
            data['firstTimestamp'] = self.first_timestamp
        if self.involved_object is not None:
            data['involvedObject'] = self.involved_object.serialize()
        if self.last_timestamp is not None:
            data['lastTimestamp'] = self.last_timestamp
        if self.message is not None:
            data['message'] = self.message
        if self.reason is not None:
            data['reason'] = self.reason
        if self.source is not None:
            data['source'] = self.source.serialize()
        if self.type is not None:
            data['type'] = self.type
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.ExecAction import ExecAction
from kubernetes.models.v1.HTTPGetAction import HTTPGetAction
from kubernetes.models.v1.TCPSocketAction import TCPSocketAction
from kubernetes.utils import filter_model


class Probe(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_probe
    """

    VALID_HANDLERS = ['exec', 'httpGet', 'tcpSocket']

    def __init__(self, model=None):
        super(Probe, self).__init__()

        self._exec_action = None
        self._http_get_action = None
        self._tcp_socket_action = None

        self._initial_delay_seconds = None
        self._timeout_seconds = None
        self._period_seconds = None
        self._success_threshold = None
        self._failure_threshold = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'exec' in model:
            self.exec_action = ExecAction(model['exec'])
        if 'httpGet' in model:
            self.http_get_action = HTTPGetAction(model['httpGet'])
        if 'tcpSocket' in model:
            self.tcp_socket_action = TCPSocketAction(model['tcpSocket'])
        if 'initialDelaySeconds' in model:
            self.initial_delay_seconds = model['initialDelaySeconds']
        if 'timeoutSeconds' in model:
            self.timeout_seconds = model['timeoutSeconds']
        if 'periodSeconds' in model:
            self.period_seconds = model['periodSeconds']
        if 'successThreshold' in model:
            self.success_threshold = model['successThreshold']
        if 'failureThreshold' in model:
            self.failure_threshold = model['failureThreshold']

    # ------------------------------------------------------------------------------------- exec

    @property
    def exec_action(self):
        return self._exec_action

    @exec_action.setter
    def exec_action(self, action=None):
        if not isinstance(action, ExecAction):
            raise SyntaxError('Probe: exec_action: [ {} ] is invalid.'.format(action))
        self._exec_action = action

    # ------------------------------------------------------------------------------------- httpGet

    @property
    def http_get_action(self):
        return self._http_get_action

    @http_get_action.setter
    def http_get_action(self, action=None):
        if not isinstance(action, HTTPGetAction):
            raise SyntaxError('Probe: http_get_action: [ {} ] is invalid.'.format(action))
        self._http_get_action = action

    # ------------------------------------------------------------------------------------- tcpSocket

    @property
    def tcp_socket_action(self):
        return self._tcp_socket_action

    @tcp_socket_action.setter
    def tcp_socket_action(self, action=None):
        if not isinstance(action, TCPSocketAction):
            raise SyntaxError('Probe: tcp_socket_action: [ {} ] is invalid.'.format(action))
        self._tcp_socket_action = action

    # ------------------------------------------------------------------------------------- initialDelaySeconds

    @property
    def initial_delay_seconds(self):
        return self._initial_delay_seconds

    @initial_delay_seconds.setter
    def initial_delay_seconds(self, secs=None):
        if not isinstance(secs, int):
            raise SyntaxError('Probe: initial_delay_seconds: [ {} ] is invalid.'.format(secs))
        self._initial_delay_seconds = secs

    # ------------------------------------------------------------------------------------- timeoutSeconds

    @property
    def timeout_seconds(self):
        return self._timeout_seconds

    @timeout_seconds.setter
    def timeout_seconds(self, secs=None):
        if not isinstance(secs, int):
            raise SyntaxError('Probe: timeout_seconds: [ {} ] is invalid.'.format(secs))
        self._timeout_seconds = secs

    # ------------------------------------------------------------------------------------- periodSeconds

    @property
    def period_seconds(self):
        return self._period_seconds

    @period_seconds.setter
    def period_seconds(self, secs=None):
        if not isinstance(secs, int):
            raise SyntaxError('Probe: period_seconds: [ {} ] is invalid.'.format(secs))
        self._period_seconds = secs

    # ------------------------------------------------------------------------------------- successThreshold

    @property
    def success_threshold(self):
        return self._success_threshold

    @success_threshold.setter
    def success_threshold(self, threshold=None):
        if not isinstance(threshold, int):
            raise SyntaxError('Probe: success_threshold: [ {} ] is invalid.'.format(threshold))
        self._success_threshold = threshold

    # ------------------------------------------------------------------------------------- failureThreshold

    @property
    def failure_threshold(self):
        return self._failure_threshold

    @failure_threshold.setter
    def failure_threshold(self, threshold=None):
        if not isinstance(threshold, int):
            raise SyntaxError('Probe: failure_threshold: [ {} ] is invalid.'.format(threshold))
        self._failure_threshold = threshold

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.exec_action is not None:
            data['exec'] = self.exec_action.serialize()
        if self.http_get_action is not None:
            data['httpGet'] = self.http_get_action.serialize()
        if self.tcp_socket_action is not None:
            data['tcpSocket'] = self.tcp_socket_action.serialize()
        if self.initial_delay_seconds is not None:
            data['initialDelaySeconds'] = self.initial_delay_seconds
        if self.timeout_seconds is not None:
            data['timeoutSeconds'] = self.timeout_seconds
        if self.period_seconds is not None:
            data['periodSeconds'] = self.period_seconds
        if self.success_threshold is not None:
            data['successThreshold'] = self.success_threshold
        if self.failure_threshold is not None:
            data['failureThreshold'] = self.failure_threshold
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model
from kubernetes.models.v1.ContainerState import ContainerState


class ContainerStatus(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_containerstatus
    """

    def __init__(self, model=None):
        super(ContainerStatus, self).__init__()

        self._name = None
        self._state = None
        self._last_state = None
        self._ready = None
        self._restart_count = None
        self._image = None
        self._image_id = None
        self._container_id = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'name' in model:
            self.name = model['name']
        if 'state' in model:
            self.state = ContainerState(model['state'])
        if 'lastState' in model:
            self.last_state = ContainerState(model['state'])
        if 'ready' in model:
            self.ready = model['ready']
        if 'restartCount' in model:
            self.restart_count = model['restartCount']
        if 'image' in model:
            self.image = model['image']
        if 'imageID' in model:
            self.image_id = model['imageID']
        if 'containerID' in model:
            self.container_id = model['containerID']

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name=None):
        if not is_valid_string(name):
            raise SyntaxError('ContainerStatus: name: [ {0} ] is invalid'.format(name))
        self._name = name

    # ------------------------------------------------------------------------------------- state

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, state=None):
        if not isinstance(state, ContainerState):
            raise SyntaxError('ContainerStatus: state: [ {0} ] is invalid'.format(state))
        self._state = state

    # ------------------------------------------------------------------------------------- lastState

    @property
    def last_state(self):
        return self._last_state

    @last_state.setter
    def last_state(self, state=None):
        if not isinstance(state, ContainerState):
            raise SyntaxError('ContainerStatus: last_state: [ {0} ] is invalid'.format(state))
        self._last_state = state

    # ------------------------------------------------------------------------------------- ready

    @property
    def ready(self):
        return self._ready

    @ready.setter
    def ready(self, ready=None):
        if not isinstance(ready, bool):
            raise SyntaxError('ContainerStatus: ready: [ {0} ] is invalid'.format(ready))
        self._ready = ready

    # ------------------------------------------------------------------------------------- restartCount

    @property
    def restart_count(self):
        return self._restart_count

    @restart_count.setter
    def restart_count(self, count=None):
        if not isinstance(count, int):
            raise SyntaxError('ContainerStatus: restart_count: [ {0} ] is invalid'.format(count))
        self._restart_count = count

    # ------------------------------------------------------------------------------------- image

    @property
    def image(self):
        return self._image

    @image.setter
    def image(self, image=None):
        if not is_valid_string(image):
            raise SyntaxError('ContainerStatus: image: [ {0} ] is invalid'.format(image))
        self._image = image

    # ------------------------------------------------------------------------------------- imageID

    @property
    def image_id(self):
        return self._image_id

    @image_id.setter
    def image_id(self, iid=None):
        if not is_valid_string(iid):
            raise SyntaxError('ContainerStatus: image_id: [ {0} ] is invalid'.format(iid))
        self._image_id = iid

    # ------------------------------------------------------------------------------------- containerID

    @property
    def container_id(self):
        return self._container_id

    @container_id.setter
    def container_id(self, cid=None):
        if not is_valid_string(cid):
            raise SyntaxError('ContainerStatus: container_id: [ {0} ] is invalid'.format(cid))
        self._container_id = cid

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.name is not None:
            data['name'] = self.name
        if self.state is not None:
            data['state'] = self.state.serialize()
        if self.last_state is not None:
            data['lastState'] = self.last_state.serialize()
        if self.ready is not None:
            data['ready'] = self.ready
        if self.restart_count is not None:
            data['restartCount'] = self.restart_count
        if self.image is not None:
            data['image'] = self.image
        if self.image_id is not None:
            data['imageID'] = self.image_id
        if self.container_id is not None:
            data['containerID'] = self.container_id
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import filter_model


class ResourceRequirements(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_resourcerequirements
    """

    VALID_RESOURCES = ['cpu', 'memory', 'storage']

    def __init__(self, model=None):
        super(ResourceRequirements, self).__init__()

        self._limits = {}
        self._requests = {}

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model):
        if 'requests' in model:
            self.requests = model['requests']
        if 'limits' in model:
            self.limits = model['limits']

    def _filter(self, data=None):
        msg = 'ResourceRequirements: data: [ {0} ] is invalid.'.format(data)
        if not isinstance(data, dict):
            raise SyntaxError(msg)
        for x in data:
            if x not in self.VALID_RESOURCES:
                data.pop(x)
        return data

    # ------------------------------------------------------------------------------------- limits

    @property
    def limits(self):
        return self._limits

    @limits.setter
    def limits(self, limits=None):
        self._limits = self._filter(limits)

    # ------------------------------------------------------------------------------------- limits

    @property
    def requests(self):
        return self._requests

    @requests.setter
    def requests(self, requests=None):
        self._requests = self._filter(requests)

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.limits:
            data['limits'] = self.limits
        if self.requests:
            data['requests'] = self.requests
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

import time

from kubernetes import K8sConfig
from kubernetes.K8sContainer import K8sContainer
from kubernetes.K8sExceptions import NotFoundException, TimedOutException
from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.Pod import Pod
from kubernetes.models.v1.PodStatus import PodStatus
from kubernetes.models.v1.Probe import Probe
from kubernetes.utils import is_valid_dict, is_valid_string, is_valid_list
from kubernetes.models.v1.Toleration import Toleration


class K8sPod(K8sObject):

    POD_READY_TIMEOUT_SECONDS = 120

    def __init__(self, config=None, name=None):
        super(K8sPod, self).__init__(
            config=config,
            obj_type='Pod',
            name=name
        )
        if self.config.pull_secret is not None:
            self.add_image_pull_secrets(self.config.pull_secret)

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sPod, self).create()
        self.get()
        self._wait_for_readiness()
        return self

    def update(self):
        super(K8sPod, self).update()
        self.get()
        self._wait_for_readiness()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sPod, self).list(labels=labels)
        pods = list(map(lambda pod: Pod(pod), ls))
        if pattern is not None:
            pods = list(filter(lambda pod: pattern in pod.name, pods))
        k8s = list()
        for x in pods:
            p = K8sPod(config=self.config, name=x.name).from_model(m=x)
            k8s.append(p)
        return k8s

    # -------------------------------------------------------------------------------------  wait

    def _wait_for_readiness(self):
        start_time = time.time()
        while not self.is_ready():
            time.sleep(0.2)
            elapsed_time = time.time() - start_time
            if elapsed_time >= self.POD_READY_TIMEOUT_SECONDS:
                raise TimedOutException(
                    "Timed out on Pod readiness: [ {0} ]".format(self.name))

    # -------------------------------------------------------------------------------------  add

    def add_container(self, container=None):
        if not isinstance(container, K8sContainer):
            raise SyntaxError(
                'K8sPod.add_container() container: [ {0} ] is invalid.'.format(container))

        containers = self.model.spec.containers
        if container not in containers:
            containers.append(container.model)
            self.model.spec.containers = containers
        return self

    def add_image_pull_secrets(self, secrets=None):
        self.model.spec.add_image_pull_secrets(secrets)
        return self

    def add_volume(self, volume=None):
        volumes = self.model.spec.volumes
        if volume not in volumes:
            volumes.append(volume.model)
        self.model.spec.volumes = volumes
        return self

    def add_toleration(self, key=None, value=None, effect=None):
        exists = False
        for tol in self.tolerations:
            if tol.key == key and tol.value == value and tol.effect == effect:
                exists = True
        if not exists:
            tol = Toleration()
            tol.key = key
            tol.value = value
            tol.effect = effect
            self.tolerations.append(tol)
        return self

    # ------------------------------------------------------------------------------------- delete

    def del_node_name(self):
        self.model.spec.node_name = None
        return self

    def del_toleration(self, key=None, value=None, effect=None):
        remaining_tolerations = []
        for tol in self.tolerations:
            if tol.key != key and tol.value != value and tol.effect != effect:
                remaining_tolerations.append(tol)
        if self.tolerations != remaining_tolerations:
            self.tolerations = remaining_tolerations
        return self

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = Pod(self.get_model())
        return self

    # ------------------------------------------------------------------------------------- polling readiness

    def is_ready(self):
        self.get()
        if self.status is not None and isinstance(self.status, PodStatus):
            pod_phase = self.status.phase
            conditions = self.status.conditions
            conditions_ok = 0
            for cond in conditions:
                if cond.status == 'True':
                    conditions_ok += 1
            if pod_phase == 'Running' and len(conditions) == conditions_ok:
                return True
        return False

    # ------------------------------------------------------------------------------------- set

    def set_container_image(self, name=None, image=None):
        containers = []
        for c in self.model.spec.containers:
            if c.name == name:
                c.image = image
            containers.append(c)
        self.model.spec.containers = containers
        return self

    # ------------------------------------------------------------------------------------- activeDeadline

    @property
    def active_deadline(self):
        return self.model.spec.active_deadline_seconds

    @active_deadline.setter
    def active_deadline(self, secs=None):
        self.model.spec.active_deadline_seconds = secs

    # ------------------------------------------------------------------------------------- automountServiceAccountToken

    @property
    def automount_service_account_token(self):
        return self.model.spec.automount_service_account_token

    @automount_service_account_token.setter
    def automount_service_account_token(self, automount=None):
        self.model.spec.automount_service_account_token = automount

    # ------------------------------------------------------------------------------------- containers

    @property
    def containers(self):
        _list = []
        for c in self.model.spec.containers:
            k8scontainer = K8sContainer(name=c.name, image=c.image)
            k8scontainer.model = c
            _list.append(k8scontainer)
        return _list

    @containers.setter
    def containers(self, containers=None):
        if not is_valid_list(containers, K8sContainer):
            self.model.spec.containers = [x.model for x in containers]

    # ------------------------------------------------------------------------------------- dnsPolicy

    @property
    def dns_policy(self):
        return self.model.spec.dns_policy

    @dns_policy.setter
    def dns_policy(self, policy=None):
        self.model.spec.dns_policy = policy

    # ------------------------------------------------------------------------------------- generateName

    @property
    def generate_name(self):
        return self.model.metadata.generate_name

    @generate_name.setter
    def generate_name(self, name=None):
        self.model.metadata.generate_name = name

    # ------------------------------------------------------------------------------------- namespace

    @property
    def namespace(self):
        return self.model.metadata.namespace

    @namespace.setter
    def namespace(self, nspace=None):
        self.model.metadata.namespace = nspace

    # ------------------------------------------------------------------------------------- nodeName

    @property
    def node_name(self):
        return self.model.spec.node_name

    @node_name.setter
    def node_name(self, name=None):
        self.model.spec.node_name = name

    # ------------------------------------------------------------------------------------- nodeSelector

    @property
    def node_selector(self):
        return self.model.spec.node_selector

    @node_selector.setter
    def node_selector(self, selector=None):
        self.model.spec.node_selector = selector

    # -------------------------------------------------------------------------------------  livenessProbes

    @property
    def liveness_probes(self):
        data = {}
        containers = self.model.spec.containers
        for c in containers:
            if c.liveness_probe is not None:
                data[c.name] = c.liveness_probe
        return data

    @liveness_probes.setter
    def liveness_probes(self, tup=None):
        if not isinstance(tup, tuple):
            raise SyntaxError(
                'K8sPod: liveness_probes: [ {} ] is invalid.'.format(tup))

        c_name, probe = tup
        container_names = [c.name for c in self.model.spec.containers]
        if c_name not in container_names:
            raise SyntaxError(
                'K8sPod: liveness_probes: container [ {} ] not found.'.format(c_name))
        if not isinstance(probe, Probe):
            raise SyntaxError(
                'K8sPod: liveness_probe: probe: [ {} ] is invalid.'.format(probe))

        containers = []
        for c in self.model.spec.containers:
            if c.name == c_name:
                c.liveness_probe = probe
            containers.append(c)
        self.model.spec.template.spec.containers = containers

    # -------------------------------------------------------------------------------------  readinessProbes

    @property
    def readiness_probes(self):
        data = {}
        containers = self.model.spec.containers
        for c in containers:
            if c.readiness_probe is not None:
                data[c.name] = c.readiness_probe
        return data

    @readiness_probes.setter
    def readiness_probes(self, tup=None):
        if not isinstance(tup, tuple):
            raise SyntaxError(
                'K8sPod: readiness_probes: [ {} ] is invalid.'.format(tup))

        c_name, probe = tup
        container_names = [c.name for c in self.model.spec.template.spec.containers]
        if c_name not in container_names:
            raise SyntaxError(
                'K8sPod: readiness_probes: container [ {} ] not found.'.format(c_name))
        if not isinstance(probe, Probe):
            raise SyntaxError(
                'K8sPod: readiness_probes: probe: [ {} ] is invalid.'.format(probe))

        containers = []
        for c in self.model.spec.template.spec.containers:
            if c.name == c_name:
                c.readiness_probe = probe
            containers.append(c)
        self.model.spec.template.spec.containers = containers

    # ------------------------------------------------------------------------------------- restartPolicy

    @property
    def restart_policy(self):
        return self.model.spec.restart_policy

    @restart_policy.setter
    def restart_policy(self, policy=None):
        self.model.spec.restart_policy = policy

    # ------------------------------------------------------------------------------------- serviceAccountName

    @property
    def service_account_name(self):
        return self.model.spec.service_account_name

    @service_account_name.setter
    def service_account_name(self, name=None):
        self.model.spec.service_account_name = name

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        self.get()
        return self.model.status

    @status.setter
    def status(self, status=None):
        self.model.status = status

    # ------------------------------------------------------------------------------------- terminationGracePeriod

    @property
    def termination_grace_period(self):
        return self.model.spec.termination_grace_period_seconds

    @termination_grace_period.setter
    def termination_grace_period(self, secs=None):
        self.model.spec.termination_grace_period_seconds = secs

    # ------------------------------------------------------------------------------------- volumes

    @property
    def volumes(self):
        return self.model.spec.volumes

    @volumes.setter
    def volumes(self, v=None):
        self.model.spec.volumes = v

    # -------------------------------------------------------------------------------------  start time

    @property
    def start_time(self):
        return self.model.status.start_time

    @start_time.setter
    def start_time(self, t=None):
        raise NotImplementedError()

    # -------------------------------------------------------------------------------------  phase

    @property
    def phase(self):
        return self.model.status.phase

    @phase.setter
    def phase(self, p=None):
        raise NotImplementedError()

    # -------------------------------------------------------------------------------------  affinity

    @property
    def affinity(self):
        return self.model.spec.affinity

    @affinity.setter
    def affinity(self, a):
        self.model.spec.affinity = a

    # -------------------------------------------------------------------------------------  tolerations

    @property
    def tolerations(self):
        return self.model.spec.tolerations

    @tolerations.setter
    def tolerations(self, t=None):
        self.model.spec.tolerations = t

    # -------------------------------------------------------------------------------------  host_ip

    @property
    def host_ip(self):
        return self.model.status.host_ip

    @host_ip.setter
    def host_ip(self, ip=None):
        raise NotImplementedError()

    # -------------------------------------------------------------------------------------  pod_ip

    @property
    def pod_ip(self):
        return self.model.status.pod_ip

    @pod_ip.setter
    def pod_ip(self, ip=None):
        raise NotImplementedError()

    # ------------------------------------------------------------------------------------- filtering

    @classmethod
    def get_by_name(cls, config=None, name=None, name_label='name'):
        if not is_valid_string(name):
            raise SyntaxError(
                'K8sPod.get_by_name(): name: [ {0} ] is invalid.'.format(name))
        return cls.get_by_labels(config=config, labels={
            name_label: name,
        })

    @staticmethod
    def get_by_labels(config=None, labels=None):
        if config is None:
            config = K8sConfig()
        if not is_valid_dict(labels):
            raise SyntaxError(
                'K8sPod.get_by_labels(): labels: [ {} ] is invalid.'.format(labels))

        pods = K8sPod(config=config, name='whatever').list(labels=labels)

        return pods

    @staticmethod
    def get_by_pod_ip(config=None, ip=None, labels=None):
        if config is None:
            config = K8sConfig()
        if not is_valid_string(ip):
            raise SyntaxError(
                'K8sPod.get_by_pod_ip(): ip: [ {0} ] is invalid.'.format(ip))

        found = None
        pods = K8sPod(config=config, name='throwaway').list(labels=labels)

        for pod in pods:
            try:
                assert isinstance(pod, K8sPod)
                if pod.pod_ip == ip:
                    found = pod
                    break
            except NotFoundException:
                pass
        return found



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.Namespace import Namespace


class K8sNamespace(K8sObject):
    def __init__(self, config=None, name=None):
        super(K8sNamespace, self).__init__(
            config=config,
            name=name,
            obj_type='Namespace'
        )

    # -------------------------------------------------------------------------------------  override

    def create(self):
        super(K8sNamespace, self).create()
        self.get()
        return self

    def update(self):
        super(K8sNamespace, self).update()
        self.get()
        return self

    def list(self, pattern=None, labels=None):
        ls = super(K8sNamespace, self).list(labels=labels)
        names = list(map(lambda x: Namespace(x), ls))
        if pattern is not None:
            names = list(filter(lambda x: pattern in x.name, names))
        k8s = []
        for x in names:
            j = K8sNamespace(config=self.config, name=x.name).from_model(m=x)
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = Namespace(self.get_model())
        return self

    def get_annotation(self, k=None):
        if k in self.model.metadata.annotations:
            return self.model.metadata.annotations[k]
        return None

    def get_label(self, k=None):
        if k in self.model.metadata.labels:
            return self.model.metadata.labels[k]
        return None

    # ------------------------------------------------------------------------------------- finalizers

    @property
    def finalizers(self):
        return self.model.spec.finalizers

    @finalizers.setter
    def finalizers(self, f=None):
        self.model.spec.finalizers = f

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self.model.status

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.metadata.name

    @name.setter
    def name(self, name=None):
        self.model.metadata.name = name

    # ------------------------------------------------------------------------------------- filter

    @staticmethod
    def get_by_name(config=None, name=None):
        namespaces = K8sNamespace(config=config, name=name).list()
        filtered = list(filter(lambda x: x.name == name, namespaces))
        if filtered:
            return filtered[0]
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.models.v1.PreferredSchedulingTerm import PreferredSchedulingTerm
from kubernetes.models.v1.NodeSelector import NodeSelector
from kubernetes.utils import is_valid_list


class NodeAffinity(object):
    """
    https://kubernetes.io/docs/api-reference/v1.6/#nodeaffinity-v1-core
    """

    def __init__(self, model=None):
        super(NodeAffinity, self).__init__()

        self._preferred_during_scheduling_ignored_during_execution = []
        self._required_during_scheduling_ignored_during_execution = None

        if model is not None:
            self._build_with_model(model)

    def _build_with_model(self, model=None):
        if 'preferredDuringSchedulingIgnoredDuringExecution' in model:
            terms = []
            for t in model['preferredDuringSchedulingIgnoredDuringExecution']:
                term = PreferredSchedulingTerm(t)
                terms.append(term)
            self.preferred_during_scheduling_ignored_during_execution = terms
        if 'requiredDuringSchedulingIgnoredDuringExecution' in model:
            sel = NodeSelector(model['requiredDuringSchedulingIgnoredDuringExecution'])
            self.required_during_scheduling_ignored_during_execution = sel

    # ----------------------------------------------------------------- preferredDuringSchedulingIgnoredDuringExecution

    @property
    def preferred_during_scheduling_ignored_during_execution(self):
        return self._preferred_during_scheduling_ignored_during_execution

    @preferred_during_scheduling_ignored_during_execution.setter
    def preferred_during_scheduling_ignored_during_execution(self, p=None):
        if not is_valid_list(p, PreferredSchedulingTerm):
            raise SyntaxError(
                'NodeAffinity: preferred_during_scheduling_ignored_during_execution: [ {} ] is invalid.'.format(p))
        self._preferred_during_scheduling_ignored_during_execution = p

    # ----------------------------------------------------------------- requiredDuringSchedulingIgnoredDuringExecution

    @property
    def required_during_scheduling_ignored_during_execution(self):
        return self._required_during_scheduling_ignored_during_execution

    @required_during_scheduling_ignored_during_execution.setter
    def required_during_scheduling_ignored_during_execution(self, r=None):
        if not isinstance(r, NodeSelector):
            raise SyntaxError(
                'NodeAffinity: required_during_scheduling_ignored_during_execution: [ {} ] is invalid.'.format(r))
        self._required_during_scheduling_ignored_during_execution = r

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.preferred_during_scheduling_ignored_during_execution:
            terms = []
            for t in self.preferred_during_scheduling_ignored_during_execution:
                term = t.serialize()
                terms.append(term)
            data['preferredDuringSchedulingIgnoredDuringExecution'] = terms
        if self.required_during_scheduling_ignored_during_execution is not None:
            sel = self.required_during_scheduling_ignored_during_execution.serialize()
            data['requiredDuringSchedulingIgnoredDuringExecution'] = sel
        return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.K8sObject import K8sObject
from kubernetes.models.v1.ComponentStatus import ComponentStatus


class K8sComponentStatus(K8sObject):
    def __init__(self, config=None, name=None):
        super(K8sComponentStatus, self).__init__(
            config=config,
            name=name,
            obj_type='ComponentStatus'
        )

    # -------------------------------------------------------------------------------------  override

    def create(self):
        return self

    def update(self):
        return self

    def delete(self, cascade=False):
        super(K8sComponentStatus, self).delete(cascade)
        return self

    def list(self, pattern=None):
        ls = super(K8sComponentStatus, self).list()
        comps = list(map(lambda x: ComponentStatus(x), ls))
        if pattern is not None:
            comps = list(filter(lambda x: pattern in x.name, comps))
        k8s = []
        for x in comps:
            j = K8sComponentStatus(config=self.config, name=x.name)
            j.model = x
            k8s.append(j)
        return k8s

    # ------------------------------------------------------------------------------------- get

    def get(self):
        self.model = ComponentStatus(self.get_model())
        return self

    # ------------------------------------------------------------------------------------- conditions

    @property
    def conditions(self):
        return self.model.conditions

    # ------------------------------------------------------------------------------------- name

    @property
    def name(self):
        return self.model.metadata.name

    @name.setter
    def name(self, name=None):
        self.model.metadata.name = name

    # ------------------------------------------------------------------------------------- filter

    @staticmethod
    def get_by_name(config=None, name=None):
        component_list = []
        components = K8sComponentStatus(config=config, name=name).list()
        for c in components:
            component_name = ComponentStatus(c).metadata.name
            if component_name == name:
                component_list.append(K8sComponentStatus(config=config, name=component_name).get())
        return component_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

from kubernetes.utils import is_valid_string, filter_model


class PodCondition(object):
    """
    http://kubernetes.io/docs/api-reference/v1/definitions/#_v1_podcondition
    """
    
    def __init__(self, model=None):
        super(PodCondition, self).__init__()

        self._last_probe_time = None
        self._last_transition_time = None
        self._message = None
        self._reason = None
        self._status = None
        self._type = None

        if model is not None:
            m = filter_model(model)
            self._build_with_model(m)

    def _build_with_model(self, model=None):
        if 'lastProbeTime' in model:
            self.last_probe_time = model['lastProbeTime']
        if 'lastTransitionTime' in model:
            self.last_transition_time = model['lastTransitionTime']
        if 'message' in model:
            self.message = model['message']
        if 'reason' in model:
            self.reason = model['reason']
        if 'status' in model:
            self.status = model['status']
        if 'type' in model:
            self.type = model['type']

    # ------------------------------------------------------------------------------------- last probe time

    @property
    def last_probe_time(self):
        return self._last_probe_time

    @last_probe_time.setter
    def last_probe_time(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('PodCondition: last_probe_time: [ {0} ] is invalid.'.format(time))
        self._last_probe_time = time

    # ------------------------------------------------------------------------------------- last transition time

    @property
    def last_transition_time(self):
        return self._last_transition_time

    @last_transition_time.setter
    def last_transition_time(self, time=None):
        if not is_valid_string(time):
            raise SyntaxError('PodCondition: last_transition_time: [ {0} ] is invalid.'.format(time))
        self._last_transition_time = time

    # ------------------------------------------------------------------------------------- message

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, msg=None):
        if not is_valid_string(msg):
            raise SyntaxError('PodCondition: message: [ {0} ] is invalid.'.format(msg))
        self._message = msg

    # ------------------------------------------------------------------------------------- reason

    @property
    def reason(self):
        return self._reason

    @reason.setter
    def reason(self, r=None):
        if not is_valid_string(r):
            raise SyntaxError('PodCondition: reasons: [ {0} ] is invalid.'.format(r))
        self._reason = r

    # ------------------------------------------------------------------------------------- status

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, status=None):
        if not is_valid_string(status):
            raise SyntaxError('PodCondition: status: [ {0} ] is invalid.'.format(status))
        self._status = status

    # ------------------------------------------------------------------------------------- type

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, t=None):
        if not is_valid_string(t):
            raise SyntaxError('PodCondition: status: [ {0} ] is invalid.'.format(t))
        self._type = t

    # ------------------------------------------------------------------------------------- serialize

    def serialize(self):
        data = {}
        if self.type is not None:
            data['type'] = self.type
        if self.status is not None:
            data['status'] = self.status
        if self.last_probe_time is not None:
            data['lastProbeTime'] = self.last_probe_time
        if self.last_transition_time is not None:
            data['lastTransitionTime'] = self.last_transition_time
        if self.reason is not None:
            data['reason'] = self.reason
        if self.message is not None:
            data['message'] = self.message
        return data
